@@ -28,7 +28,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-
+#include <regex.h>
+#include <stdio_ext.h>
 
 static int debug = 0;
 
@@ -51,45 +52,47 @@ static int setup_log_file_array(auparse_state_t *au)
 
         /* Load config so we know where logs are */
 	set_aumessage_mode(MSG_STDERR, DBG_NO);
-        load_config(&config, TEST_SEARCH);
+	load_config(&config, TEST_SEARCH);
 
-        /* for each file */
-        len = strlen(config.log_file) + 16;
-        filename = malloc(len);
-        if (!filename) {
-                fprintf(stderr, "No memory\n");
-                free_config(&config);
-                return 1;
-        }
-        /* Find oldest log file */
-        snprintf(filename, len, "%s", config.log_file);
-        do {
-                if (access(filename, R_OK) != 0)
-                        break;
-                num++;
-                snprintf(filename, len, "%s.%d", config.log_file, num);
-        } while (1);
-        num--;
+	/* for each file */
+	len = strlen(config.log_file) + 16;
+	filename = malloc(len);
+	if (!filename) {
+		fprintf(stderr, "No memory\n");
+		free_config(&config);
+		return 1;
+	}
+	/* Find oldest log file */
+	snprintf(filename, len, "%s", config.log_file);
+	do {
+		if (access(filename, R_OK) != 0)
+			break;
+		num++;
+		snprintf(filename, len, "%s.%d", config.log_file, num);
+	} while (1);
+	num--;
 	tmp = malloc((num+2)*sizeof(char *));
 
 
         /* Got it, now process logs from last to first */
-        if (num > 0)
-                snprintf(filename, len, "%s.%d", config.log_file, num);
-        else
-                snprintf(filename, len, "%s", config.log_file);
-        do {
+	if (num > 0)
+		snprintf(filename, len, "%s.%d", config.log_file, num);
+	else
+		snprintf(filename, len, "%s", config.log_file);
+	do {
 		tmp[i++] = strdup(filename);
 
-                /* Get next log file */
-                num--;
-                if (num > 0)
-                        snprintf(filename, len, "%s.%d", config.log_file, num);
-                else if (num == 0)
-                        snprintf(filename, len, "%s", config.log_file);
-                else
-                        break;
-        } while (1);
+		/* Get next log file */
+		num--;
+		if (num > 0)
+			snprintf(filename, len, "%s.%d", config.log_file, num);
+		else if (num == 0)
+			snprintf(filename, len, "%s", config.log_file);
+		else
+			break;
+	} while (1);
+	free_config(&config);
+	free(filename);
 
 	// Terminate the list
 	tmp[i] = NULL; 
@@ -221,17 +224,23 @@ void auparse_add_callback(auparse_state_t *au, auparse_callback_ptr callback,
 static void consume_feed(auparse_state_t *au, int flush)
 {
 	while (auparse_next_event(au) > 0) {
-		if (au->callback) (*au->callback)(au, AUPARSE_CB_EVENT_READY, au->callback_user_data);
+		if (au->callback) {
+			(*au->callback)(au, AUPARSE_CB_EVENT_READY,
+					au->callback_user_data);
+		}
 	}
 	if (flush) {
-		// FIXME: We might have to add a call here to force auparse_next_event()
+		// FIXME: might need a call here to force auparse_next_event()
 		// to consume any partial data not fully consumed.
 		if (au->parse_state == EVENT_ACCUMULATING) {
 			// Emit the event, set event cursors to initial position
 			aup_list_first(&au->le);
 			aup_list_first_field(&au->le);
 			au->parse_state = EVENT_EMITTED;
-			if (au->callback) (*au->callback)(au, AUPARSE_CB_EVENT_READY, au->callback_user_data);
+			if (au->callback) {
+				 (*au->callback)(au, AUPARSE_CB_EVENT_READY,
+						 au->callback_user_data);
+			}
 		}
 	}
 }
@@ -307,12 +316,13 @@ static int lookup_op(const char *op)
 }
 
 
-int ausearch_add_item(auparse_state_t *au, const char *field, const char *op,
-	const char *value, ausearch_rule_t how)
+static int ausearch_add_item_internal(auparse_state_t *au, const char *field,
+	const char *op, const char *value, ausearch_rule_t how,
+	ausearch_op_t op_flag)
 {
 	aurulenode rn;
 	int t_op = lookup_op(op);
-	if (op < 0)
+	if (t_op < 0)
 		goto err_out;
 
 	// Make sure there's a field
@@ -335,15 +345,85 @@ int ausearch_add_item(auparse_state_t *au, const char *field, const char *op,
 	}
 
 	// All pre-checks are done, build a rule
-	rn.search_op = t_op;
+	rn.search_op = t_op | op_flag;
 	if (t_op == AUSEARCH_EXISTS)
-		rn.val = NULL;
+		rn.v.field.val = NULL;
 	else {
 		if (value == NULL)
 			goto err_out;
-		rn.val = strdup(value);
+		rn.v.field.val = strdup(value);
 	}
-	rn.field = strdup(field);
+	rn.v.field.field = strdup(field);
+	aurule_append(&au->rules, &rn);
+	au->search_how = how;
+
+	return 0;
+err_out:
+	errno = EINVAL;
+	return -1;
+}
+
+int ausearch_add_item(auparse_state_t *au, const char *field, const char *op,
+	const char *value, ausearch_rule_t how)
+{
+	return ausearch_add_item_internal(au, field, op, value, how, 0);
+}
+
+int ausearch_add_interpreted_item(auparse_state_t *au, const char *field,
+	const char *op, const char *value, ausearch_rule_t how)
+{
+	return ausearch_add_item_internal(au, field, op, value, how,
+					  AUSEARCH_INTERPRETED);
+}
+
+static int lookup_ts_op(const char *op)
+{
+	static const struct nv_pair ts_tab[] = {
+		{AUSEARCH_TIME_LT, "<"},
+		{AUSEARCH_TIME_LE, "<="},
+		{AUSEARCH_TIME_GE, ">="},
+		{AUSEARCH_TIME_GT, ">"},
+		{AUSEARCH_TIME_EQ, "="},
+	};
+
+        size_t i;
+
+        for (i = 0; i < sizeof(ts_tab) / sizeof(*ts_tab); i++)
+                if (strcmp(ts_tab[i].name, op) == 0)
+                        return ts_tab[i].value;
+
+        return -1;
+}
+
+int ausearch_add_timestamp_item(auparse_state_t *au, const char *op, time_t sec,
+				unsigned milli, ausearch_rule_t how)
+{
+	aurulenode rn;
+	int t_op = lookup_ts_op(op);
+	if (op < 0)
+		goto err_out;
+	if (milli >= 1000)
+		goto err_out;
+
+	// Do not allow regex to get replaced this way
+	if (au->search_how == AUSEARCH_RULE_REGEX)
+		goto err_out;
+
+	// Make sure how is within range
+	if (how < AUSEARCH_RULE_CLEAR || how > AUSEARCH_RULE_AND)
+		goto err_out;
+	if (how == AUSEARCH_RULE_CLEAR) {
+		aurule_clear(&au->rules);
+	} else if (how != au->search_how
+		   && au->search_how != AUSEARCH_RULE_CLEAR) {
+		errno = EEXIST;
+		return -1;
+	}
+
+	// All pre-checks are done, build a rule
+	rn.search_op = t_op;
+	rn.v.timestamp.sec = sec;
+	rn.v.timestamp.milli = milli;
 	aurule_append(&au->rules, &rn);
 	au->search_how = how;
 
@@ -355,25 +435,30 @@ err_out:
 
 int ausearch_add_regex(auparse_state_t *au, const char *expr)
 {
-//	aurulenode rn;
+	aurulenode rn;
+	int rc;
 
 	// Make sure there's an expression
 	if (expr == NULL)
 		goto err_out;
 
-	if (au->search_how == AUSEARCH_RULE_REGEX)
+	if (au->search_how != AUSEARCH_RULE_CLEAR)
 		goto err_out;
 
-	// FIXME: Need to do a parse or something to make sure the
-	// expression is legal
+	// Compile expression now to make sure the expression is correct
+	rc = regcomp(&au->regex, expr, REG_EXTENDED|REG_NOSUB);
+	if (rc) {
+		goto err_out;
+	}
 
-/*	rn.field = strdup(expr);
+	// Store it away just in case.
 	rn.search_op = AUSEARCH_UNSET;
-	rn.val = NULL;
+	rn.v.field.field = strdup(expr);
+	rn.v.field.val = NULL;
 	aurule_append(&au->rules, &rn);
+	au->search_how = AUSEARCH_RULE_REGEX;
 
-	au->search_how = how;
-	return 0; */
+	return 0; 
 
 err_out:
 	errno = EINVAL;
@@ -393,6 +478,8 @@ int ausearch_set_stop(auparse_state_t *au, austop_t where)
 
 void ausearch_clear(auparse_state_t *au)
 {
+	if (au->search_how == AUSEARCH_RULE_REGEX && au->rules.cnt > 0)
+		regfree(&au->regex);
 	aurule_clear(&au->rules);
 	au->search_where = AUSEARCH_STOP_EVENT;
 	au->search_how = AUSEARCH_RULE_CLEAR;
@@ -463,7 +550,8 @@ static int readline_file(auparse_state_t *au)
 	}
 	if ((rc = getline(&au->cur_buf, &n, au->in)) <= 0) {
 		// Note: getline always malloc's if lineptr==NULL or n==0,
-		// on failure malloc'ed memory is left uninitialized, caller must free it.
+		// on failure malloc'ed memory is left uninitialized,
+		// caller must free it.
 		free(au->cur_buf);
 		au->cur_buf = NULL;
 
@@ -503,7 +591,6 @@ static int readline_file(auparse_state_t *au)
  *    -1 if error (errno contains non-zero error code)
  *    -2 if EOF  (errno == 0)
  */
- 
 
 static int readline_buf(auparse_state_t *au)
 {
@@ -527,11 +614,13 @@ static int readline_buf(auparse_state_t *au)
 		
 		/* dup the line */
 		au->cur_buf = malloc(line_len+1);   // +1 for null terminator
-		if (au->cur_buf == NULL) return -1; // return error condition, errno set
+		if (au->cur_buf == NULL)
+			return -1; // return error condition, errno set
 		strncpy(au->cur_buf, databuf_beg(&au->databuf), line_len);
 		au->cur_buf[line_len] = 0;
 
-		if (databuf_advance(&au->databuf, line_len+1) < 0) return -1;
+		if (databuf_advance(&au->databuf, line_len+1) < 0)
+			return -1;
 		// return success
 		errno = 0;
 		return 1;
@@ -570,13 +659,21 @@ static int str2event(char *s, au_event_t *e)
 	return 0;
 }
 
-static void extract_timestamp(const char *b, au_event_t *e)
+/* Returns 0 on success and 1 on error */
+static int extract_timestamp(const char *b, au_event_t *e)
 {
 	char *ptr, *tmp;
+	int rc = 1;
 
+        e->host = NULL;
 	tmp = strndupa(b, 80);
 	ptr = strtok(tmp, " ");
 	if (ptr) {
+		// Optionally grab the node - may or may not be included
+		if (strncmp(ptr, "node=", 5 ) == 0) {
+			e->host = strdup(ptr+5);
+			ptr = strtok(NULL, " "); // Bump along to the next one
+		}
 		// at this point we have type=
 		ptr = strtok(NULL, " ");
 		if (ptr) {
@@ -592,21 +689,34 @@ static void extract_timestamp(const char *b, au_event_t *e)
 				if (eptr)
 					*eptr = 0;
 
-				if (str2event(ptr, e)) {
+				if (str2event(ptr, e) == 0)
+					rc = 0;
+//				else {
 //					audit_msg(LOG_ERROR,
 //					  "Error extracting time stamp (%s)\n",
 //						ptr);
-				}
+//				}
 			}
 			// else we have a bad line
 		}
 		// else we have a bad line
 	}
 	// else we have a bad line
+	return rc;
 }
 
 static int events_are_equal(au_event_t *e1, au_event_t *e2)
 {
+	// If both have a host, only a string compare can tell if they
+	// are the same. Otherwise, if only one of them have a host, they
+	// are definitely not the same. Its a boundary on daemon config.
+	if (e1->host && e2->host) {
+		if (strcmp(e1->host, e2->host))
+			return 0;
+	} else if (e1->host || e2->host)
+		return 0;
+
+	// Hosts are same at this point, check time & serial
 	if (e1->serial == e2->serial && e1->milli == e2->milli &&
 			e1->sec == e2->sec)
 		return 1;
@@ -659,24 +769,34 @@ static int retrieve_next_line(auparse_state_t *au)
 			// if the first time through, open file
 			if (au->list_idx == 0 && au->in == NULL) {
 				au->line_number = 0;
-				au->in = fopen(au->source_list[au->list_idx], "r");
-				if (au->in == NULL) return -1;
+				au->in = fopen(au->source_list[au->list_idx],
+									"r");
+				if (au->in == NULL)
+					return -1;
+				__fsetlocking(au->in, FSETLOCKING_BYCALLER);
 			}
 
 			// loop reading lines from a file
 			while (au->in) {
 				if ((rc = readline_file(au)) == -2) {
-					// end of file, open next file, try readline again
+					// end of file, open next file,
+					// try readline again
 					fclose(au->in);
 					au->in = NULL;
 					au->list_idx++;
 					au->line_number = 0;
 					if (au->source_list[au->list_idx]) {
-						au->in = fopen(au->source_list[au->list_idx], "r");
-						if (au->in == NULL) return -1;
+						au->in = fopen(
+						  au->source_list[au->list_idx],
+						  "r");
+						if (au->in == NULL)
+							return -1;
+						__fsetlocking(au->in,
+							FSETLOCKING_BYCALLER);
 					}
 				} else {
-					if (rc > 0) au->line_number++;
+					if (rc > 0)
+						au->line_number++;
 					return rc;
 				}
 			}
@@ -684,15 +804,18 @@ static int retrieve_next_line(auparse_state_t *au)
 		case AUSOURCE_BUFFER:
 		case AUSOURCE_BUFFER_ARRAY:
 			rc = readline_buf(au);
-			if (rc > 0) au->line_number++;
+			if (rc > 0)
+				au->line_number++;
 			return rc;
 		case AUSOURCE_FEED:
 			rc = readline_buf(au);
-			// No such thing as EOF for feed, translate EOF to data not available
+			// No such thing as EOF for feed, translate EOF
+			// to data not available
 			if (rc == -2)
 				return 0;
 			else
-				if (rc > 0) au->line_number++;
+				if (rc > 0)
+					au->line_number++;
 				return rc;
 		default:
 			return -1;
@@ -732,6 +855,62 @@ static int ausearch_reposition_cursors(auparse_state_t *au)
 	return rc;
 }
 
+/* Return the result of interpreting rule on the event record r. */
+static int ausearch_interpret_rule(auparse_state_t *au, rnode *r,
+				   aurulenode *rule)
+{
+	int search_op;
+
+	search_op = (rule->search_op & ~AUSEARCH_INTERPRETED);
+	if (AUSEARCH_OP_IS_TIME__(search_op)) {
+		int val;
+
+		if (au->le.e.sec < rule->v.timestamp.sec)
+			val = -1;
+		else if (au->le.e.sec > rule->v.timestamp.sec)
+			val = 1;
+		else if (au->le.e.milli < rule->v.timestamp.milli)
+			val = -1;
+		else if (au->le.e.milli > rule->v.timestamp.milli)
+			val = 1;
+		else
+			val = 0;
+		switch (search_op) {
+		case AUSEARCH_TIME_LT:
+			return val < 0;
+		case AUSEARCH_TIME_LE:
+			return val <= 0;
+		case AUSEARCH_TIME_GE:
+			return val >= 0;
+		case AUSEARCH_TIME_GT:
+			return val > 0;
+		case AUSEARCH_TIME_EQ:
+			return val == 0;
+		default:
+			abort();
+		}
+	} else {
+		int found, rc;
+		const char *val;
+
+		found = nvlist_find_name(&r->nv, rule->v.field.field);
+		if (!found)
+			return 0;
+		if (search_op == AUSEARCH_EXISTS)
+			return 1;
+
+		if ((rule->search_op & AUSEARCH_INTERPRETED) != 0)
+			val = nvlist_interp_cur_val(r);
+		else
+			val = nvlist_get_cur_val(&r->nv);
+		rc = strcmp(rule->v.field.val, val);
+		if (search_op == AUSEARCH_EQUAL)
+			return rc == 0;
+		else
+			return rc != 0;
+	}
+}
+
 /* This is called during search once per each record. It walks the list
  * of nvpairs and decides if a field matches. */
 static int ausearch_compare(auparse_state_t *au)
@@ -739,9 +918,18 @@ static int ausearch_compare(auparse_state_t *au)
 	rnode *r;
 
 	if (au->search_how == AUSEARCH_RULE_REGEX) {
-		// FIXME: need to code this one
-		errno = ENOSYS;
-		return -1;
+		int rc;
+
+		if (au->rules.cnt == 0) {
+			errno = EINVAL;
+			return -1;
+		}
+		r = aup_list_get_cur(&au->le);
+		rc = regexec(&au->regex, r->record, 0, NULL, 0);
+		if (rc == 0)
+			return 1;
+		else
+			return 0;
 	}
 	r = aup_list_get_cur(&au->le);
 	if (r) {
@@ -749,22 +937,12 @@ static int ausearch_compare(auparse_state_t *au)
 		int results = (au->search_how == AUSEARCH_RULE_AND);
 		aurule_first(&au->rules);
 		do {
-			int rc, tmp = 0;
+			int tmp;
 
 			aurulenode *rule = aurule_get_cur(&au->rules);
 			if (rule == NULL) 
 				continue;
-			rc = nvlist_find_name(&r->nv, rule->field);
-			if (rc) {
-				if (rule->search_op == AUSEARCH_EXISTS)
-					return 1;
-				rc = strcmp(rule->val,
-					    nvlist_get_cur_val(&r->nv));
-				if (rule->search_op == AUSEARCH_EQUAL)
-					tmp = (rc == 0);
-				else
-					tmp = (rc != 0);
-			}
+			tmp = ausearch_interpret_rule(au, r, rule);
 			if (au->search_how == AUSEARCH_RULE_AND) {
 				if (tmp == 0)
 					return 0; /* all must match */
@@ -814,7 +992,8 @@ int auparse_next_event(auparse_state_t *au)
 
 	if (au->parse_state == EVENT_EMITTED) {
 		// If the last call resulted in emitting event data then
-		// clear previous event data in preparation to accumulate new event data
+		// clear previous event data in preparation to accumulate
+		// new event data
 		aup_list_clear(&au->le);
 		au->parse_state = EVENT_EMPTY;
 	}
@@ -826,7 +1005,8 @@ int auparse_next_event(auparse_state_t *au)
 		if (rc ==  0) return 0;	// No data now
 		if (rc == -2) {
 			// We're at EOF, did we read any data previously?
-			// If so return data available, else return no data available
+			// If so return data available, else return no data
+			// available
 			if (au->parse_state == EVENT_ACCUMULATING) {
 				if (debug) printf("EOF, EVENT_EMITTED\n");
 				au->parse_state = EVENT_EMITTED;
@@ -835,30 +1015,47 @@ int auparse_next_event(auparse_state_t *au)
 			return 0;
 		}
 		if (rc > 0) {	        // Input available
-			extract_timestamp(au->cur_buf, &event);
+			if (extract_timestamp(au->cur_buf, &event)) {
+				if (debug)
+					printf("Malformed line:%s\n",
+							 au->cur_buf);
+				continue;
+			}
 			if (au->parse_state == EVENT_EMPTY) {
 				// First record in new event, initialize event
-				if (debug) printf("First record in new event, initialize event\n");
+				if (debug)
+					printf(
+			"First record in new event, initialize event\n");
 				aup_list_set_event(&au->le, &event);
-				aup_list_append(&au->le, au->cur_buf, au->list_idx, au->line_number);
+				aup_list_append(&au->le, au->cur_buf,
+						au->list_idx, au->line_number);
 				au->parse_state = EVENT_ACCUMULATING;
 				au->cur_buf = NULL; 
+			// FIXME: Need to see if hosts are the same, too.
 			} else if (events_are_equal(&au->le.e, &event)) {
 				// Accumulate data into existing event
-				if (debug) printf("Accumulate data into existing event\n");
-				aup_list_append(&au->le, au->cur_buf, au->list_idx, au->line_number);
+				if (debug)
+					printf(
+				    "Accumulate data into existing event\n");
+				aup_list_append(&au->le, au->cur_buf,
+						au->list_idx, au->line_number);
 				au->parse_state = EVENT_ACCUMULATING;
 				au->cur_buf = NULL; 
 			} else {
-				// New event, save current input for next invocation
-				if (debug) printf("New event, save current input for next invocation, EVENT_EMITTED\n");
+				// New event, save input for next invocation
+				if (debug)
+					printf(
+	"New event, save current input for next invocation, EVENT_EMITTED\n");
 				push_line(au);
-				// Emit the event, set event cursors to initial position
+				// Emit the event, set event cursors to 
+				// initial position
 				aup_list_first(&au->le);
 				aup_list_first_field(&au->le);
 				au->parse_state = EVENT_EMITTED;
+				free((char *)event.host);
 				return 1; // data is available
 			}
+			free((char *)event.host);
 		} else {		// Read error
 			return -1;
 		}
@@ -906,8 +1103,26 @@ unsigned long auparse_get_serial(auparse_state_t *au)
 // Gets the machine node name
 const char *auparse_get_node(auparse_state_t *au)
 {
-// FIXME Need to do this one
-	return NULL;
+	if (au && au->le.e.host != NULL)
+		return strdup(au->le.e.host);
+	else
+		return NULL;
+}
+
+
+int auparse_node_compare(au_event_t *e1, au_event_t *e2)
+{
+	// If both have a host, only a string compare can tell if they
+	// are the same. Otherwise, if only one of them have a host, they
+	// are definitely not the same. Its a boundary on daemon config.
+	if (e1->host && e2->host) 
+		return strcmp(e1->host, e2->host);
+	else if (e1->host)
+		return 1;
+	else if (e2->host)
+		return -1;
+
+	return 0;
 }
 
 
