@@ -916,7 +916,7 @@ static const struct transtab cloneflagtab[] = {
 static const char *print_clone_flags(int flags)
 {
         int i, cnt = 0;
-	char buf[192];
+	char buf[352];
 
 	buf[0] = 0;
         for (i=0; i<CLONE_FLAG_NAMES; i++) {
@@ -978,9 +978,20 @@ static const char *print_fcntl_cmd(int cmd)
 static const char *print_a0(const char *val, const rnode *r)
 {
 	int machine = r->machine, syscall = r->syscall;
+	char *out;
 	const char *sys = audit_syscall_to_name(syscall, machine);
 	if (sys) {
-		/* Unused right now... */
+		if (strcmp(sys, "clone") == 0) {
+			int ival;
+
+			errno = 0;
+			ival = strtoul(val, NULL, 16);
+		        if (errno) {
+                		asprintf(&out, "conversion error(%s)", val);
+	                	return out;
+	        	}
+			return print_clone_flags(ival);
+		}
 	}
 	return strdup(val);
 }
@@ -1022,17 +1033,7 @@ static const char *print_a2(const char *val, const rnode *r)
 	char *out;
 	const char *sys = audit_syscall_to_name(syscall, machine);
 	if (sys) {
-		if (strcmp(sys, "clone") == 0) {
-			int ival;
-
-			errno = 0;
-			ival = strtoul(val, NULL, 16);
-		        if (errno) {
-                		asprintf(&out, "conversion error(%s)", val);
-	                	return out;
-	        	}
-			return print_clone_flags(ival);
-		} else if (strncmp(sys, "fcntl", 5) == 0) {
+		if (strncmp(sys, "fcntl", 5) == 0) {
 			int ival;
 
 			errno = 0;
@@ -1045,6 +1046,10 @@ static const char *print_a2(const char *val, const rnode *r)
 			{
 				case F_SETOWN:
 					return print_uid(val);
+				case F_SETFD:
+					if (ival == FD_CLOEXEC)
+						return strdup("FD_CLOEXEC");
+					/* Fall thru okay. */
 				case F_SETFL:
 				case F_SETLEASE:
 				case F_GETLEASE:
@@ -1106,6 +1111,16 @@ static int audit_lookup_type(const char *name)
 	return -1;
 }
 
+static int is_hex_string(const char *str)
+{
+	while (*str) {
+		if (!isxdigit(*str))
+			return 0;
+		str++;
+	}
+	return 1;
+}
+
 const char *interpret(const rnode *r)
 {
 	const nvlist *nv = &r->nv;
@@ -1115,10 +1130,21 @@ const char *interpret(const rnode *r)
 	const char *name = nvlist_get_cur_name(nv);
 	const char *val = nvlist_get_cur_val(nv);
 
+	/* Do some fixups */
 	if (r->type == AUDIT_EXECVE && name[0] == 'a')
 		type = T_ESCAPED;
-	else
+	else if (r->type == AUDIT_AVC && strcmp(name, "saddr") == 0)
+		type = -1;
+	else if (strcmp(name, "acct") == 0) {
+		if (val[0] == '"')
+			type = T_ESCAPED;
+		else if (is_hex_string(val))
+			type = T_ESCAPED;
+		else
+			type = -1;
+	} else
 		type = audit_lookup_type(name);
+
 	switch(type) {
 		case T_UID:
 			out = print_uid(val);
