@@ -28,21 +28,27 @@ static void walk_test(auparse_state_t *au)
 						strerror(errno));
 			exit(1);
 		}
-		printf("event %d has %d records\n", event_cnt, auparse_get_num_records(au));
+		printf("event %d has %d records\n", event_cnt,
+						auparse_get_num_records(au));
 		record_cnt = 1;
 		do {
 			printf("    record %d of type %d(%s) has %d fields\n",
-			       record_cnt, 
-			       auparse_get_type(au), audit_msg_type_to_name(auparse_get_type(au)),
-			       auparse_get_num_fields(au));
+				record_cnt, 
+				auparse_get_type(au),
+				audit_msg_type_to_name(auparse_get_type(au)),
+				auparse_get_num_fields(au));
 			printf("    line=%d file=%s\n",
-			       auparse_get_line_number(au), auparse_get_filename(au) ? auparse_get_filename(au) : "None");
+				auparse_get_line_number(au),
+				auparse_get_filename(au) ?
+					auparse_get_filename(au) : "None");
 			const au_event_t *e = auparse_get_timestamp(au);
 			if (e == NULL) {
 				printf("Error getting timestamp - aborting\n");
 				exit(1);
 			}
-			printf("    event time: %u.%u:%lu\n", (unsigned)e->sec, e->milli, e->serial);
+			printf("    event time: %u.%u:%lu, host=%s\n",
+				(unsigned)e->sec,
+				e->milli, e->serial, e->host ? e->host : "?");
 			auparse_first_field(au);
 			do {
 				printf("        %s=%s (%s)\n",
@@ -70,17 +76,23 @@ void light_test(auparse_state_t *au)
 		record_cnt = 1;
 		do {
 			printf("    record %d of type %d(%s) has %d fields\n",
-			       record_cnt, 
-			       auparse_get_type(au), audit_msg_type_to_name(auparse_get_type(au)),
-			       auparse_get_num_fields(au));
+				record_cnt, 
+				auparse_get_type(au),
+				audit_msg_type_to_name(auparse_get_type(au)),
+				auparse_get_num_fields(au));
 			printf("    line=%d file=%s\n",
-			       auparse_get_line_number(au), auparse_get_filename(au) ? auparse_get_filename(au) : "None");
+				auparse_get_line_number(au),
+				auparse_get_filename(au) ?
+					auparse_get_filename(au) : "None");
 			const au_event_t *e = auparse_get_timestamp(au);
 			if (e == NULL) {
 				printf("Error getting timestamp - aborting\n");
 				exit(1);
 			}
-			printf("    event time: %u.%u:%lu\n", (unsigned)e->sec, e->milli, e->serial);
+			printf("    event time: %u.%u:%lu, host=%s\n",
+					(unsigned)e->sec,
+					e->milli, e->serial,
+					e->host ? e->host : "?");
 			printf("\n");
 			record_cnt++;
 		} while(auparse_next_record(au) > 0);
@@ -177,11 +189,33 @@ void compound_search(ausearch_rule_t how)
 	auparse_destroy(au);
 }
 
-static int random_num(int max)
+void regex_search(const char *expr)
 {
-	int j;
-	j = 1 + (int) ((float)max * (rand() / (RAND_MAX + 1.0)));
-	return j;
+	auparse_state_t *au;
+	int rc;
+
+	au = auparse_init(AUSOURCE_BUFFER_ARRAY, buf);
+	if (au == NULL) {
+		printf("auparse_init error - %s\n", strerror(errno));
+		exit(1);
+	}
+	if (ausearch_add_regex(au, expr)){
+		printf("ausearch_add_regex error - %s\n", strerror(errno));
+		exit(1);
+	}
+	if (ausearch_set_stop(au, AUSEARCH_STOP_RECORD)){
+		printf("ausearch_set_stop error - %s\n", strerror(errno));
+		exit(1);
+	}
+	rc = ausearch_next_event(au);
+	if (rc < 0)
+		printf("Error searching for %s - %s\n", expr, strerror(errno));
+	else if (rc == 0)
+		printf("Not found\n");
+	else
+		printf("Found %s = %s\n", auparse_get_field_name(au),
+					auparse_get_field_str(au));
+	auparse_destroy(au);
 }
 
 static void auparse_callback(auparse_state_t *au, auparse_cb_event_t cb_event_type, void *user_data)
@@ -194,20 +228,27 @@ static void auparse_callback(auparse_state_t *au, auparse_cb_event_t cb_event_ty
 			printf("can't get first record\n");
 			return;
 		}
-		printf("event %d has %d records\n", *event_cnt, auparse_get_num_records(au));
+		printf("event %d has %d records\n", *event_cnt,
+					auparse_get_num_records(au));
 		record_cnt = 1;
 		do {
 			printf("    record %d of type %d(%s) has %d fields\n",
-			       record_cnt, 
-			       auparse_get_type(au), audit_msg_type_to_name(auparse_get_type(au)),
-			       auparse_get_num_fields(au));
+				record_cnt, 
+				auparse_get_type(au),
+				audit_msg_type_to_name(auparse_get_type(au)),
+				auparse_get_num_fields(au));
 			printf("    line=%d file=%s\n",
-			       auparse_get_line_number(au), auparse_get_filename(au) ? auparse_get_filename(au) : "None");
+				auparse_get_line_number(au),
+				auparse_get_filename(au) ?
+					auparse_get_filename(au) : "None");
 			const au_event_t *e = auparse_get_timestamp(au);
 			if (e == NULL) {
 				return;
 			}
-			printf("    event time: %u.%u:%lu\n", (unsigned)e->sec, e->milli, e->serial);
+			printf("    event time: %u.%u:%lu, host=%s\n",
+					(unsigned)e->sec,
+					e->milli, e->serial, 
+					e->host ? e->host : "?");
 			auparse_first_field(au);
 			do {
 				printf("        %s=%s (%s)\n",
@@ -343,26 +384,45 @@ int main(void)
 	auparse_destroy(au);
 	printf("Test 7 Done\n\n");
 
+	printf("Starting Test 8, regex search...\n");
+	au = auparse_init(AUSOURCE_BUFFER_ARRAY, buf);
+	if (au == NULL) {
+		printf("Error - %s\n", strerror(errno));
+		return 1;
+	}
+	puts("Doing regex match...");
+	regex_search("1143146623");
+	auparse_destroy(au);
+	au = auparse_init(AUSOURCE_BUFFER_ARRAY, buf);
+	puts("Doing regex wildcard search...");
+	regex_search("11431466.*146");
+	auparse_destroy(au);
+	printf("Test 8 Done\n\n");
 
 	/* Note: this should match Test 2 exactly */
-	printf("Starting Test 8, buffer feed...\n");
+	printf("Starting Test 9, buffer feed...\n");
 	{
 		int event_cnt = 1;
 		size_t len, chunk_len = 3;
-		const char **cur_buf, *p_beg, *p_end, *p_chunk_beg, *p_chunk_end;
+		const char **cur_buf, *p_beg, *p_end, *p_chunk_beg,
+			*p_chunk_end;
 
 		au = auparse_init(AUSOURCE_FEED, 0);
 		auparse_add_callback(au, auparse_callback, &event_cnt, NULL);
-		for (cur_buf = buf, p_beg = *cur_buf; *cur_buf; cur_buf++, p_beg = *cur_buf) {
+		for (cur_buf = buf, p_beg = *cur_buf; *cur_buf;
+						 cur_buf++, p_beg = *cur_buf) {
 			len = strlen(p_beg);
 			p_end = p_beg + len;
 			p_chunk_beg = p_beg;
 			while (p_chunk_beg < p_end) {
 				p_chunk_end = p_chunk_beg + chunk_len;
-				if (p_chunk_end > p_end) p_chunk_end = p_end;
+				if (p_chunk_end > p_end)
+					p_chunk_end = p_end;
 
-				//fwrite(p_chunk_beg, 1, p_chunk_end-p_chunk_beg, stdout);
-				auparse_feed(au, p_chunk_beg, p_chunk_end-p_chunk_beg);
+				//fwrite(p_chunk_beg, 1,
+				//	 p_chunk_end-p_chunk_beg, stdout);
+				auparse_feed(au, p_chunk_beg,
+						p_chunk_end-p_chunk_beg);
 				p_chunk_beg = p_chunk_end;
 			}
 		}
@@ -370,10 +430,10 @@ int main(void)
 		auparse_flush_feed(au);
 		auparse_destroy(au);
 	}
-		printf("Test 8 Done\n\n");
+		printf("Test 9 Done\n\n");
 
 		/* Note: this should match Test 4 exactly */
-		printf("Starting Test 9, file feed...\n");
+		printf("Starting Test 10, file feed...\n");
 	{
 		int *event_cnt = malloc(sizeof(int));
 		size_t len;
@@ -385,7 +445,8 @@ int main(void)
 		au = auparse_init(AUSOURCE_FEED, 0);
 		auparse_add_callback(au, auparse_callback, event_cnt, free);
 		if ((fp = fopen(filename, "r")) == NULL) {
-			fprintf(stderr, "could not open '%s', %s\n", filename, strerror(errno));
+			fprintf(stderr, "could not open '%s', %s\n",
+						filename, strerror(errno));
 			return 1;
 		}
 		while ((len = fread(buf, 1, sizeof(buf), fp))) {
@@ -396,13 +457,13 @@ int main(void)
 		auparse_flush_feed(au);
 		auparse_destroy(au);
 	}
-        printf("Test 9 Done\n\n");
+        printf("Test 10 Done\n\n");
 
 	if (getuid() != 0) {
 		puts("Finished non-admin tests\n");
 		return 0;
 	}
-	printf("Starting Test 10, walk events, records of logs...\n");
+	printf("Starting Test 11, walk events, records of logs...\n");
 	au = auparse_init(AUSOURCE_LOGS, NULL);
 	if (au == NULL) {
 		printf("Error - %s\n", strerror(errno));
@@ -410,7 +471,7 @@ int main(void)
 	}
 	light_test(au);
 	auparse_destroy(au);
-	printf("Test 10 Done\n\n");
+	printf("Test 11 Done\n\n");
 
 	return 0;
 }
