@@ -57,8 +57,8 @@ AuEvent_get_sec(AuEvent *self, void *closure)
 {
     if (self->sec == NULL) {
         if ((self->sec = PyInt_FromLong(self->event.sec)) == NULL) return NULL;
-        Py_INCREF(self->sec);
     }
+    Py_INCREF(self->sec);
     return self->sec;
 }
 
@@ -67,8 +67,8 @@ AuEvent_get_milli(AuEvent *self, void *closure)
 {
     if (self->milli == NULL) {
         if ((self->milli = PyInt_FromLong(self->event.milli)) == NULL) return NULL;
-        Py_INCREF(self->milli);
     }
+    Py_INCREF(self->milli);
     return self->milli;
 }
 
@@ -77,8 +77,8 @@ AuEvent_get_serial(AuEvent *self, void *closure)
 {
     if (self->serial == NULL) {
         if ((self->serial = PyInt_FromLong(self->event.serial)) == NULL) return NULL;
-        Py_INCREF(self->serial);
     }
+    Py_INCREF(self->serial);
     return self->serial;
 }
 
@@ -88,7 +88,9 @@ AuEvent_get_host(AuEvent *self, void *closure)
     if (self->event.host == NULL) {
         Py_RETURN_NONE;
     } else {
-        if ((self->host = PyString_FromString(self->event.host)) == NULL) return NULL;
+	if (self->host == NULL) {
+	    if ((self->host = PyString_FromString(self->event.host)) == NULL) return NULL;
+	}
         Py_INCREF(self->host);
         return self->host;
     }
@@ -597,6 +599,8 @@ is equal to the value given in this rule.\n\
 locate the field name and check that the value associated with\n\
 it is NOT equal to the value given in this rule.\n\
 \n\
+The value parameter is compared to the uninterpreted field value.\n\
+\n\
 The how parameter determines how this search condition will be stored\n\
 internally. The possible values are:\n\
 \n\
@@ -636,6 +640,136 @@ AuParser_search_add_item(AuParser *self, PyObject *args)
     return NULL;
 }
 
+/********************************
+ * ausearch_add_interpreted_item
+ ********************************/
+PyDoc_STRVAR(search_add_interpreted_item_doc,
+"search_add_interpreted_item(field, op, value, how) Build up search rule\n\
+\n\
+\n\
+search_add_interpreted_item() adds one search condition to the audit search\n\
+API. The search conditions can then be used to scan logs, files, or\n\
+buffers for something of interest. The field value is the field name\n\
+that the value will be checked for. The op variable describes what\n\
+kind of check is to be done. Legal op values are:\n\
+\n\
+'exists':\n\
+Just check that a field name exists\n\
+\n\
+'=':\n\
+locate the field name and check that the value associated with it\n\
+is equal to the value given in this rule.\n\
+\n\
+'!=':\n\
+locate the field name and check that the value associated with\n\
+it is NOT equal to the value given in this rule.\n\
+\n\
+The value parameter is compared to the interpreted field value (the value\n\
+that would be returned by AuParser.interpret_field).\n\
+\n\
+The how parameter determines how this search condition will be stored\n\
+internally. The possible values are:\n\
+\n\
+AUSEARCH_RULE_CLEAR:\n\
+When this is used, it clears any previous search condition and inserts\n\
+it as the first one.\n\
+\n\
+AUSEARCH_RULE_OR:\n\
+\n\
+When this is used, it means that the results of its evaluation will be\n\
+or'ed with other search conditions.\n\
+\n\
+AUSEARCH_RULE_AND:\n\
+When this is used, it means that the results of its evaluation will be\n\
+and'ed with other search conditions.\n\
+\n\
+All search conditions must be the same type, you cannot mix 'and' and 'or'.\n\
+\n\
+No Return value, raises exception (EnvironmentError) on error.\n\
+");
+
+static PyObject *
+AuParser_search_add_interpreted_item(AuParser *self, PyObject *args)
+{
+    const char *field;
+    const char *op;
+    const char *value;
+    int how;
+    int result;
+
+    if (!PyArg_ParseTuple(args, "sssi", &field, &op, &value, &how)) return NULL;
+    PARSER_CHECK;
+
+    result = ausearch_add_interpreted_item(self->au, field, op, value, how);
+    if (result == 0) Py_RETURN_NONE;
+    PyErr_SetFromErrno(PyExc_EnvironmentError);
+    return NULL;
+}
+
+/********************************
+ * ausearch_add_timestamp_item
+ ********************************/
+PyDoc_STRVAR(search_add_timestamp_item_doc,
+"search_add_timestamp_item(op, sec, milli, how) Build up search rule\n\
+\n\
+\n\
+search_add_timestamp_item adds an event time condition to the audit\n\
+search state. The search conditions can then be used to scan logs, files,\n\
+or buffers for something of interest. The op parameter specifies the\n\
+desired comparison. Legal op values are \"<\", \"<=\", \">=\", \">\" and\n\
+\"=\". The left operand of the comparison operator is the timestamp of the\n\
+examined event, the right operand is specified by the sec and milli\n\
+parameters.\n\
+\n\
+The how parameter determines how this search condition will be stored\n\
+internally. The possible values are:\n\
+\n\
+AUSEARCH_RULE_CLEAR:\n\
+When this is used, it clears any previous search condition and inserts\n\
+it as the first one.\n\
+\n\
+AUSEARCH_RULE_OR:\n\
+\n\
+When this is used, it means that the results of its evaluation will be\n\
+or'ed with other search conditions.\n\
+\n\
+AUSEARCH_RULE_AND:\n\
+When this is used, it means that the results of its evaluation will be\n\
+and'ed with other search conditions.\n\
+\n\
+All search conditions must be the same type, you cannot mix 'and' and 'or'.\n\
+\n\
+No Return value, raises exception (EnvironmentError) on error.\n\
+");
+
+static PyObject *
+AuParser_search_add_timestamp_item(AuParser *self, PyObject *args)
+{
+    const char *op;
+    PY_LONG_LONG sec;
+    int milli;
+    int how;
+    int result;
+
+    /* There's no completely portable way to handle time_t values from Python;
+       note that time_t might even be a floating-point type!  PY_LONG_LONG
+       is at least enough not to worry about year 2038.
+
+       milli is int because Python's 'I' format does no overflow checking.
+       Negative milli values will wrap to values > 1000 and
+       ausearch_add_timestamp_item will reject them. */
+    if (!PyArg_ParseTuple(args, "sLii", &op, &sec, &milli, &how))
+	    return NULL;
+    PARSER_CHECK;
+
+    result = ausearch_add_timestamp_item(self->au, op, sec, (unsigned)milli,
+					 how);
+    if (result == 0)
+	    Py_RETURN_NONE;
+    PyErr_SetFromErrno(PyExc_EnvironmentError);
+    return NULL;
+}
+
 /********************************
  * ausearch_add_regex
  ********************************/
@@ -1238,6 +1372,8 @@ static PyMethodDef AuParser_methods[] = {
     {"add_callback",      (PyCFunction)AuParser_add_callback,      METH_VARARGS, add_callback_doc},
     {"reset",             (PyCFunction)AuParser_reset,             METH_NOARGS,  reset_doc},
     {"search_add_item",   (PyCFunction)AuParser_search_add_item,   METH_VARARGS, search_add_item_doc},
+    {"search_add_interpreted_item", (PyCFunction)AuParser_search_add_interpreted_item, METH_VARARGS, search_add_interpreted_item_doc},
+    {"search_add_timestamp_item", (PyCFunction)AuParser_search_add_timestamp_item, METH_VARARGS, search_add_timestamp_item_doc},
     {"search_add_regex",  (PyCFunction)AuParser_search_add_regex,  METH_VARARGS, search_add_regex_doc},
     {"search_set_stop",   (PyCFunction)AuParser_search_set_stop,   METH_VARARGS, search_set_stop_doc},
     {"search_clear",      (PyCFunction)AuParser_search_clear,      METH_NOARGS,  search_clear_doc},
