@@ -1,5 +1,5 @@
 /* audit_logging.c -- 
- * Copyright 2005,2006 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-2007 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -30,11 +30,13 @@
 #include <netinet/in.h> // inet6 addrlen
 #include <netdb.h>	// gethostbyname
 #include <arpa/inet.h>	// inet_ntop
+#include <utmp.h>
 
 #include "libaudit.h"
 #include "private.h"
 
 #define TTY_PATH	32
+#define MAX_USER	(UT_NAMESIZE * 2) + 8
 
 // NOTE: The kernel fills in pid, uid, and loginuid of sender. Therefore,
 // these routines do not need to send them.
@@ -60,7 +62,7 @@ static void _resolve_addr(char buf[], const char *host)
 
 	e = getaddrinfo(host, NULL, &hints, &ai);
 	if (e != 0) {
-		audit_msg(LOG_ERR,
+		audit_msg(LOG_ERR, 
 			"resolve_addr: cannot resolve hostname %s (%s)",
 			host, gai_strerror(e));
 		return;
@@ -120,7 +122,7 @@ static char *_get_exename(char *exename, int size)
  * Get the command line name 
  * NOTE: at the moment, this only escapes what the user sent
  */
-static char *_get_commname(const char *comm, char *commname, int size)
+static char *_get_commname(const char *comm, char *commname, unsigned int size)
 {
 	const unsigned char *p = (const unsigned char *)comm;
 	int found = 0;
@@ -217,6 +219,7 @@ int audit_log_user_message(int audit_fd, int type, const char *message,
 		_resolve_addr(addrbuf, hostname);
 	else
 		strncat(addrbuf, addr, sizeof(addrbuf)-1);
+
 	_get_exename(exename, sizeof(exename));
 	if (tty == NULL) 
 		tty = _get_tty(ttyname, TTY_PATH);
@@ -231,7 +234,7 @@ int audit_log_user_message(int audit_fd, int type, const char *message,
 		);
 
 	errno = 0;
-	ret = audit_send_user_message( audit_fd, type, buf );
+	ret = audit_send_user_message( audit_fd, type, HIDE_IT, buf );
 	if ((ret < 1) && errno == 0)
 		errno = ret;
 	return ret;
@@ -295,7 +298,7 @@ int audit_log_user_comm_message(int audit_fd, int type, const char *message,
 		);
 
 	errno = 0;
-	ret = audit_send_user_message( audit_fd, type, buf );
+	ret = audit_send_user_message( audit_fd, type, HIDE_IT, buf );
 	if ((ret < 1) && errno == 0)
 		errno = ret;
 	return ret;
@@ -354,16 +357,40 @@ int audit_log_acct_message(int audit_fd, int type, const char *pgname,
 	if (tty == NULL) 
 		tty = _get_tty(ttyname, TTY_PATH);
 
-	if (name)
-		snprintf(buf, sizeof(buf), 
-	     "op=%s acct=%s exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)",
-			op, name, pgname,
+	if (name) {
+		char user[MAX_USER];
+		const char *format, *p;
+		size_t len;
+		int enc = 0;
+
+		user[0] = 0;
+		strncat(user, name, MAX_USER-1);
+		len = strnlen(user, UT_NAMESIZE);
+		user[len] = 0;
+		p = user;
+		while (*p) {
+			if (*p == '"' || *p < 0x21 || (unsigned)*p > 0x7f) {
+				_audit_c2x(user, name, len);
+				enc = 1;
+				break;
+			}
+			p++;
+		}
+		if (enc)
+			format = 
+	 "op=%s acct=\"%s\" exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)";
+		else
+			format = 
+	     "op=%s acct=%s exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)";
+
+		snprintf(buf, sizeof(buf), format,
+			op, user, pgname,
 			host ? host : "?",
 			addrbuf,
 			tty ? tty : "?",
 			success
 			);
-	else
+	} else
 		snprintf(buf, sizeof(buf),
 		"op=%s id=%u exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)",
 			op, id, pgname,
@@ -374,7 +401,7 @@ int audit_log_acct_message(int audit_fd, int type, const char *pgname,
 			);
 
 	errno = 0;
-	ret = audit_send_user_message( audit_fd, type, buf );
+	ret = audit_send_user_message(audit_fd, type, REAL_ERR, buf);
 	if ((ret < 1) && errno == 0)
 		errno = ret;
 	return ret;
@@ -382,8 +409,8 @@ int audit_log_acct_message(int audit_fd, int type, const char *pgname,
 
 /*
  * This function will log a message to the audit system using a predefined
- * message format. This function should be used by all console apps that do
- * not manipulate accounts or groups.
+ * message format. This function should be used by all apps that are SE Linux
+ * object managers.
  *
  * audit_fd - The fd returned by audit_open
  * type - type of message, ex: AUDIT_USER, AUDIT_USYS_CONFIG, AUDIT_USER_LOGIN
@@ -403,7 +430,6 @@ int audit_log_user_avc_message(int audit_fd, int type, const char *message,
 	char exename[PATH_MAX*2];
 	char ttyname[TTY_PATH];
 	int retval;
-	int ret;
 
 	if (audit_fd < 0)
 		return 0;
@@ -426,7 +452,7 @@ int audit_log_user_avc_message(int audit_fd, int type, const char *message,
 		);
 
 	errno = 0;
-	retval = audit_send_user_message( audit_fd, type, buf );
+	retval = audit_send_user_message( audit_fd, type, REAL_ERR, buf );
 	if (retval == -EPERM && getuid() != 0) {
 		syslog(LOG_ERR, "Can't send to audit system: %s %s",
 			audit_msg_type_to_name(type), buf);
@@ -437,3 +463,228 @@ int audit_log_user_avc_message(int audit_fd, int type, const char *message,
 	return retval;
 }
 
+/*
+ * This function will log a message to the audit system using a predefined
+ * message format. It should be used for all SE linux user and role 
+ * manipulation operations.
+ * Parameter usage is as follows:
+ *
+ * type - type of message: AUDIT_USER_ROLE_CHANGE for changing any SE Linux
+ *        user or role attributes.
+ * pgname - program's name
+ * op  -  operation. "adding user", "changing finger info", "deleting group"
+ * name - user's account or group name. If not available use NULL.
+ * id  -  uid or gid that the operation is being performed on. This is used
+ *        only when user is NULL.
+ * new_seuser - the new seuser that the login user is getting
+ * new_role - the new_role that the login user is getting
+ * new_range - the new mls range that the login user is getting
+ * old_seuser - the old seuser that the login usr had
+ * old_role - the old role that the login user had
+ * old_range - the old mls range that the login usr had
+ * host - The hostname if known
+ * addr - The network address of the user
+ * tty  - The tty of the user
+ * result - 1 is "success" and 0 is "failed"
+ *
+ * It returns the sequence number which is > 0 on success or <= 0 on error.
+ */
+int audit_log_semanage_message(int audit_fd, int type, const char *pgname,
+	const char *op, const char *name, unsigned int id, 
+	const char *new_seuser, const char *new_role, const char *new_range,
+	const char *old_seuser, const char *old_role, const char *old_range,
+	const char *host, const char *addr,
+	const char *tty, int result)
+{
+	const char *success;
+	char buf[MAX_AUDIT_MESSAGE_LENGTH];
+	char addrbuf[INET6_ADDRSTRLEN];
+	char exename[PATH_MAX*2];
+	char ttyname[TTY_PATH];
+	int ret;
+
+	if (audit_fd < 0)
+		return 0;
+
+	if (result)
+		success = "success";
+	else
+		success = "failed";
+
+	addrbuf[0] = 0;
+	if (addr == NULL || strlen(addr) == 0)
+		_resolve_addr(addrbuf, host);
+	else
+		strncat(addrbuf, addr, sizeof(addrbuf)-1);
+
+	if (pgname == NULL || strlen(pgname) == 0) {
+		_get_exename(exename, sizeof(exename));
+		pgname = exename;
+	}
+	if (tty == NULL || strlen(tty) == 0) 
+		tty = _get_tty(ttyname, TTY_PATH);
+
+	if (name && strlen(name) > 0) {
+		int enc = 0;
+		size_t len;
+		const char *format, *p = name;
+		char user[MAX_USER];
+
+		user[0] = 0;
+		strncat(user, name, MAX_USER-1);
+		len = strnlen(user, UT_NAMESIZE);
+		user[len] = 0;
+		while (*p) {
+			if (*p == '"' || *p < 0x21 || (unsigned)*p > 0x7f) {
+				_audit_c2x(user, name, len);
+				enc = 1;
+				break;
+			}
+			p++;
+		}
+		if (enc)
+			format = "op=%s acct=%s old-seuser=%s old-role=%s old-range=%s new-seuser=%s new-role=%s new-range=%s exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)";
+		else
+			format = "op=%s acct=\"%s\" old-seuser=%s old-role=%s old-range=%s new-seuser=%s new-role=%s new-range=%s exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)";
+		snprintf(buf, sizeof(buf), format, op, user, 
+			old_seuser && strlen(old_seuser) ? old_seuser : "?",
+			old_role && strlen(old_role) ? old_role : "?",
+			old_range && strlen(old_range) ? old_range : "?",
+			new_seuser && strlen(new_seuser) ? new_seuser : "?",
+			new_role && strlen(new_role) ? new_role : "?",
+			new_range && strlen(new_range) ? new_range : "?",
+			pgname,
+			host && strlen(host) ? host : "?",
+			addrbuf,
+			tty && strlen(tty) ? tty : "?",
+			success
+			);
+	} else
+		snprintf(buf, sizeof(buf),
+		"op=%s id=%u old seuser=%s old role=%s old range=%s new seuser=%s new role=%s new range=%s exe=%s (hostname=%s, addr=%s, terminal=%s res=%s)",
+			op, id,
+			old_seuser && strlen(old_seuser) ? old_seuser : "?",
+			old_role && strlen(old_role) ? old_role : "?",
+			old_range && strlen(old_range) ? old_range : "?",
+			new_seuser && strlen(new_seuser) ? new_seuser : "?",
+			new_role && strlen(new_role) ? new_role : "?",
+			new_range && strlen(new_range) ? new_range : "?",
+			pgname,
+			host && strlen(host) ? host : "?",
+			addrbuf,
+			tty && strlen(tty) ? tty : "?",
+			success
+			);
+
+	errno = 0;
+	ret = audit_send_user_message(audit_fd, type, REAL_ERR, buf);
+	if ((ret < 1) && errno == 0)
+		errno = ret;
+	return ret;
+}
+
+/*
+ * This function will log a message to the audit system using a predefined
+ * message format. This function should be used by all console apps that do
+ * not manipulate accounts or groups.
+ *
+ * audit_fd - The fd returned by audit_open
+ * type - type of message, ex: AUDIT_USER_CMD
+ * command - the command line being logged
+ * tty - The tty of the user
+ * result - 1 is "success" and 0 is "failed"
+ *
+ * It returns the sequence number which is > 0 on success or <= 0 on error.
+ */
+int audit_log_user_command(int audit_fd, int type, const char *command,
+	const char *tty, int result)
+{
+	char *p;
+	char buf[MAX_AUDIT_MESSAGE_LENGTH];
+	char commname[PATH_MAX*2];
+	char cwdname[PATH_MAX*2];
+	char ttyname[TTY_PATH];
+	char format[64];
+	const char *success;
+	char *cmd;
+	int ret,cwdenc=0, cmdenc=0;
+	unsigned int len;
+
+	if (audit_fd < 0)
+		return 0;
+
+	if (result)
+		success = "success";
+	else
+		success = "failed";
+
+	if (tty == NULL) 
+		tty = _get_tty(ttyname, TTY_PATH);
+
+	/* Trim leading spaces */
+	while (*command == ' ')
+		command++;
+
+	cmd = strdup(command);
+	if (cmd == NULL)
+		return -1;
+
+	// We borrow the commname buffer
+	if (getcwd(commname, PATH_MAX) == NULL)
+		strcpy(cwdname, "?");
+	strcpy(cwdname, commname);
+	p = commname;
+	len = strlen(commname);
+	while (*p) {
+		if (*p == '"' || *p < 0x21 || (unsigned)*p > 0x7f) {
+			_audit_c2x(cwdname, commname, len);
+			cwdenc = 1;
+			break;
+		}
+		p++;
+	}
+
+	len = strlen(cmd);
+	// Trim the trailing carriage return and spaces
+	while (len && (cmd[len-1] == 0x0A || cmd[len-1] == ' ')) {
+		cmd[len-1] = 0;
+		len--;
+	}
+
+	p = cmd;
+	strcpy(commname, cmd);
+	while (*p) {
+		if (*p == '"' || *p < 0x21 || (unsigned)*p > 0x7f) {
+			_audit_c2x(commname, cmd, len);
+			cmdenc = 1;
+			break;
+		}
+		p++;
+	}
+
+	if (cwdenc)
+		p=stpcpy(format, "cwd=%s ");
+	else
+		p=stpcpy(format, "cwd=\"%s\" ");
+
+	if (cmdenc)
+		p = stpcpy(p, "cmd=%s ");
+	else
+		p = stpcpy(p, "cmd=\"%s\" ");
+
+	strcpy(p, "(terminal=%s res=%s)");
+
+	snprintf(buf, sizeof(buf), format,
+			cwdname, commname,
+			tty ? tty : "?",
+			success
+		);
+
+	free(cmd);
+	errno = 0;
+	ret = audit_send_user_message( audit_fd, type, HIDE_IT, buf );
+	if ((ret < 1) && errno == 0)
+		errno = ret;
+	return ret;
+}
+
