@@ -641,20 +641,19 @@ int audit_make_equivalent(int fd, const char *mount_point,
 	int rc;
 	size_t len1 = strlen(mount_point);
 	size_t len2 = strlen(subtree);
-	struct audit_rule_data *rule = malloc(sizeof(struct audit_rule_data) +
-						len1 + len2);
-	
-	memset(rule, 0, sizeof(*rule) + len1 + len2);
+ 	struct {
+ 		uint32_t sizes[2];
+ 		unsigned char buf[];
+ 	} *cmd = malloc(sizeof(*cmd) + len1 + len2);
 
-	rule->field_count = 2;
-	rule->buflen = len1 + len2;
-	rule->values[0] = len1;
-	memcpy(&rule->buf[0], mount_point, len1);
-	rule->values[1] = len2;
-	memcpy(&rule->buf[1], subtree, len2);
+ 	memset(cmd, 0, sizeof(*cmd) + len1 + len2);
 
-	rc = audit_send(fd, AUDIT_MAKE_EQUIV, rule, 
-			sizeof(struct audit_rule_data) + rule->buflen);
+ 	cmd->sizes[0] = len1;
+ 	cmd->sizes[1] = len2;
+ 	memcpy(&cmd->buf[0], mount_point, len1);
+ 	memcpy(&cmd->buf[len1], subtree, len2);
+
+ 	rc = audit_send(fd, AUDIT_MAKE_EQUIV, cmd, sizeof(*cmd) + len1 + len2);
 	if (rc < 0) 
 		audit_msg(audit_priority(errno),
 			"Error sending make_equivalent command (%s)",
@@ -867,6 +866,22 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 				}
 			}
 			break;
+		case AUDIT_EXIT:
+			vlen = strlen(v);
+			if (isdigit((char)*(v))) 
+				rule->values[rule->field_count] = 
+					strtol(v, NULL, 0);
+			else if (vlen >= 2 && *(v)=='-' && 
+						(isdigit((char)*(v+1)))) 
+				rule->values[rule->field_count] = 
+					strtol(v, NULL, 0);
+			else {
+				rule->values[rule->field_count] = 
+						audit_name_to_errno(v);
+				if (rule->values[rule->field_count] == 0) 
+					return -15;
+			}
+			break;
 		case AUDIT_MSGTYPE:
 			if (flags != AUDIT_FILTER_EXCLUDE)
 				return -9;
@@ -1040,10 +1055,10 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 					}
 				}
 				rule->values[rule->field_count] = val;
-				audit_syscalladded = 1;// perm selects syscalls
 			}
 			break;
-		case AUDIT_DEVMAJOR...AUDIT_SUCCESS:
+		case AUDIT_DEVMAJOR...AUDIT_INODE:
+		case AUDIT_SUCCESS...AUDIT_SUCCESS:
 			if (flags == AUDIT_FILTER_ENTRY)
 				return -7;
 			/* fallthrough */
