@@ -81,11 +81,14 @@ extern "C" {
 #define AUDIT_TRUSTED_APP	1121	/* Trusted app msg - freestyle text */
 #define AUDIT_USER_SELINUX_ERR	1122	/* SE Linux user space error */
 #define AUDIT_USER_CMD		1123	/* User shell command and args */
+#define AUDIT_USER_TTY		1124	/* Non-ICANON TTY input meaning */
+#define AUDIT_CHUSER_ID		1125	/* Changed user ID supplemental data */
 
 #define AUDIT_FIRST_DAEMON	1200
 #define AUDIT_LAST_DAEMON	1299
 #define AUDIT_DAEMON_RECONFIG	1204	/* Audit daemon should reconfigure */
 #define AUDIT_DAEMON_ROTATE	1205	/* Audit daemon should rotate logs */
+#define AUDIT_DAEMON_RESUME	1206	/* Audit daemon should resume logging */
 
 #define AUDIT_FIRST_EVENT	1300
 #define AUDIT_LAST_EVENT	1399
@@ -103,7 +106,13 @@ extern "C" {
 #define AUDIT_FD_PAIR		1317	/* audit record for pipe/socketpair */
 #endif
 #ifndef AUDIT_OBJ_PID
-#define AUDIT_OBJ_PID		1318	/* Ptrace target */
+#define AUDIT_OBJ_PID		1318	/* signal or ptrace target */
+#endif
+#ifndef AUDIT_TTY
+#define AUDIT_TTY		1319	/* Input on an administrative TTY */
+#endif
+#ifndef AUDIT_EOE
+#define AUDIT_EOE		1320	/* End of event */
 #endif
 #define AUDIT_LAST_EVENT	1399
 
@@ -131,8 +140,14 @@ extern "C" {
 #endif
 #define AUDIT_FIRST_APPARMOR		1500
 #define AUDIT_LAST_APPARMOR		1599
-#ifndef AUDIT_SD
-#define AUDIT_SD			1500	/* Not upstream yet */
+#ifndef AUDIT_AA
+#define AUDIT_AA			1500	/* Not upstream yet */
+#define AUDIT_APPARMOR_AUDIT		1501
+#define AUDIT_APPARMOR_ALLOWED		1502
+#define AUDIT_APPARMOR_DENIED		1503
+#define AUDIT_APPARMOR_HINT		1504
+#define AUDIT_APPARMOR_STATUS		1505
+#define AUDIT_APPARMOR_ERROR		1506
 #endif
 
 #define AUDIT_FIRST_KERN_CRYPTO_MSG	1600
@@ -210,6 +225,12 @@ extern "C" {
 #define AUDIT_MAKE_EQUIV        1015    /* Append to watched tree */
 #endif
 
+/* These are from the audit by tty patch */
+#ifndef AUDIT_TTY_GET
+#define AUDIT_TTY_GET		1016	/* Get TTY auditing status */
+#define AUDIT_TTY_SET		1017	/* Set TTY audit status */
+#endif
+
 /* This is for the new operator patch */
 #ifndef AUDIT_BIT_MASK
 #define AUDIT_BIT_MASK			0x08000000
@@ -443,6 +464,8 @@ extern const char *audit_machine_to_name(int machine);
 extern unsigned int audit_machine_to_elf(int machine);
 extern int          audit_elf_to_machine(unsigned int elf);
 extern const char *audit_operator_to_symbol(int op);
+extern int        audit_name_to_errno(const char *error);
+extern const char *audit_errno_to_name(int error);
 
 /* AUDIT_GET */
 extern int audit_request_status(int fd);
