@@ -1,5 +1,5 @@
 /* msg_typetab.h --
- * Copyright 2005,2006 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-07 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -39,6 +39,8 @@ _S(AUDIT_LOGIN,                      "LOGIN"                         )
 _S(AUDIT_LIST_RULES,                 "LIST_RULES"                    )
 //_S(AUDIT_TRIM,                     "TRIM"                          )
 //_S(AUDIT_MAKE_EQUIV,               "MAKE_EQUIV"                    )
+_S(AUDIT_TTY_GET,                    "TTY_GET"                       )
+_S(AUDIT_TTY_SET,                    "TTY_SET"                       )
 _S(AUDIT_USER_AUTH,                  "USER_AUTH"                     )
 _S(AUDIT_USER_ACCT,                  "USER_ACCT"                     )
 _S(AUDIT_USER_MGMT,                  "USER_MGMT"                     )
@@ -63,12 +65,15 @@ _S(AUDIT_TEST,                       "TEST"                          )
 _S(AUDIT_TRUSTED_APP,                "TRUSTED_APP"                   )
 _S(AUDIT_USER_SELINUX_ERR,           "USER_SELINUX_ERR"              )
 _S(AUDIT_USER_CMD,                   "USER_CMD"                      )
+_S(AUDIT_USER_TTY,                   "USER_TTY"                      )
+_S(AUDIT_CHUSER_ID,                  "CHUSER_ID"                     )
 _S(AUDIT_DAEMON_START,               "DAEMON_START"                  )
 _S(AUDIT_DAEMON_END,                 "DAEMON_END"                    )
 _S(AUDIT_DAEMON_ABORT,               "DAEMON_ABORT"                  )
 _S(AUDIT_DAEMON_CONFIG,              "DAEMON_CONFIG"                 )
 //_S(AUDIT_DAEMON_RECONFIG,          "DAEMON_RECONFIG"               )
 _S(AUDIT_DAEMON_ROTATE,              "DAEMON_ROTATE"                 )
+_S(AUDIT_DAEMON_RESUME,              "DAEMON_RESUME"                 )
 _S(AUDIT_SYSCALL,                    "SYSCALL"                       )
 //_S(AUDIT_FS_WATCH,                 "FS_WATCH"                      )
 _S(AUDIT_PATH,                       "PATH"                          )
@@ -87,6 +92,8 @@ _S(AUDIT_MQ_GETSETATTR,              "MQ_GETSETATTR"                 )
 _S(AUDIT_KERNEL_OTHER,               "KERNEL_OTHER"                  )
 _S(AUDIT_FD_PAIR,                    "FD_PAIR"                       )
 _S(AUDIT_OBJ_PID,                    "OBJ_PID"                       )
+_S(AUDIT_TTY,                        "TTY"                           )
+_S(AUDIT_EOE,                        "EOE"                           )
 _S(AUDIT_AVC,                        "AVC"                           )
 _S(AUDIT_SELINUX_ERR,                "SELINUX_ERR"                   )
 _S(AUDIT_AVC_PATH,                   "AVC_PATH"                      )
@@ -105,7 +112,13 @@ _S(AUDIT_MAC_IPSEC_DELSPD,           "MAC_IPSEC_DELSPD"              )
 _S(AUDIT_ANOM_PROMISCUOUS,           "ANOM_PROMISCUOUS"              )
 _S(AUDIT_ANOM_ABEND,                 "ANOM_ABEND"                    )
 #ifdef WITH_APPARMOR
-_S(AUDIT_SD,                         "APPARMOR"                      )
+_S(AUDIT_AA,                         "APPARMOR"                      )
+_S(AUDIT_APPARMOR_AUDIT,             "APPARMOR_AUDIT"                )
+_S(AUDIT_APPARMOR_ALLOWED,           "APPARMOR_ALLOWED"              )
+_S(AUDIT_APPARMOR_DENIED,            "APPARMOR_DENIED"               )
+_S(AUDIT_APPARMOR_HINT,              "APPARMOR_HINT"                 )
+_S(AUDIT_APPARMOR_STATUS,            "APPARMOR_STATUS"               )
+_S(AUDIT_APPARMOR_ERROR,             "APPARMOR_ERROR"                )
 #endif
 _S(AUDIT_KERNEL,                     "KERNEL"                        )
 _S(AUDIT_ANOM_LOGIN_FAILURES,        "ANOM_LOGIN_FAILURES"           )
