@@ -148,7 +148,15 @@ static int adjust_reply(struct audit_reply *rep, int len)
 	rep->signal_info = NULL;
 	rep->conf     = NULL;
 	if (!NLMSG_OK(rep->nlh, (unsigned int)len)) {
-		errno = EBADE;
+		if (len == sizeof(rep->msg)) {
+			audit_msg(LOG_ERR, 
+				"Netlink event from kernel is too big");
+			errno = EFBIG;
+		} else {
+			audit_msg(LOG_ERR, 
+			"Netlink message from kernel was not OK");
+			errno = EBADE;
+		}
 		return 0;
 	}
 
