@@ -1,5 +1,5 @@
 /* s390_table.h --
- * Copyright 2005,2006 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-07 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -305,4 +305,9 @@ _S(309, "vmsplice")
 _S(311, "getcpu")
 _S(312, "epoll_pwait")
 _S(313, "utimes")
+_S(314, "fallocate")
+_S(315, "utimensat")
+_S(316, "signalfd")
+_S(317, "timerfd")
+_S(318, "eventfd")
 
