@@ -269,4 +269,9 @@ _S(309, "vmsplice")
 _S(311, "getcpu")
 _S(312, "epoll_pwait")
 _S(313, "utimes")
+_S(314, "fallocate")
+_S(315, "utimensat")
+_S(316, "signalfd")
+_S(317, "timerfd")
+_S(318, "eventfd")
 
