@@ -70,6 +70,7 @@ static struct audit_rule_data *rule_new = NULL;
 extern int audit_archadded;
 extern int audit_syscalladded;
 extern unsigned int audit_elf;
+int audit_permadded;
 
 /*
  * This function will reset everything used for each loop when loading 
@@ -79,6 +80,7 @@ static int reset_vars(void)
 {
 	list_requested = 0;
 	audit_syscalladded = 0;
+	audit_permadded = 0;
 	audit_archadded = 0;
 	audit_elf = 0;
 	add = AUDIT_FILTER_UNSET;
@@ -289,8 +291,10 @@ static int audit_setup_perms(struct audit_rule_data *rule, const char *opt)
 		}
 	}
 
-	if (audit_update_watch_perms(rule, val) == 0)
+	if (audit_update_watch_perms(rule, val) == 0) {
+		audit_permadded = 1;
 		return 1;
+	}
 	return -1;
 }
 
@@ -327,7 +331,7 @@ void audit_request_rule_list(int fd)
 }
 // FIXME: Change these to enums
 /*
- * returns: -3 depreacted, -2 success - no reply, -1 error - noreply,
+ * returns: -3 deprecated, -2 success - no reply, -1 error - noreply,
  * 0 success - reply, > 0 success - rule
  */
 static int setopt(int count, char *vars[])
@@ -584,6 +588,14 @@ static int setopt(int count, char *vars[])
 		switch (rc)
 		{
 			case 0:
+				if (which == OLD && 
+				    	rule.fields[rule.field_count-1] ==
+						AUDIT_PERM)
+					audit_permadded = 1;
+				else if (which == NEW &&
+				    rule_new->fields[rule_new->field_count-1] ==
+						AUDIT_PERM)
+					audit_permadded = 1;
 				break;
 			case -1:
 				fprintf(stderr, "-F missing = for %s\n", 
@@ -661,6 +673,11 @@ static int setopt(int count, char *vars[])
 					optarg);
 				retval = -1;
 				break;
+			case -15:
+				fprintf(stderr, 
+					"-F unknown errno - %s\n", optarg);
+				retval = -1;
+				break;
 			default:
 				retval = -1;
 				break;
@@ -715,7 +732,7 @@ static int setopt(int count, char *vars[])
 		}
 		break;
 	case 'k':
-		if (audit_syscalladded != 1 ||
+		if (!(audit_syscalladded || audit_permadded ) ||
 				(add==AUDIT_FILTER_UNSET &&
 					del==AUDIT_FILTER_UNSET)) {
 			fprintf(stderr,
@@ -765,7 +782,7 @@ static int setopt(int count, char *vars[])
 				fprintf(stderr,
 				"You must give a watch prior to perms\n");
 				retval = -1;
-			} else
+			} else 
 				retval = audit_setup_perms(rule_new, optarg);
 		}
 		break;
@@ -1016,8 +1033,26 @@ static int handle_request(int status)
 			if (which == OLD) {
 				rc = audit_add_rule(fd, &rule, add, action);
 			} else {
+				set_aumessage_mode(MSG_QUIET, DBG_NO);
 				rc = audit_add_rule_data(fd, rule_new,
 								 add, action);
+				set_aumessage_mode(MSG_STDERR, DBG_NO);
+				/* Retry for legacy kernels */
+				if (rc < 0) {
+					if (errno == EINVAL &&
+					rule_new->fields[0] == AUDIT_DIR) {
+						rule_new->fields[0] =
+								AUDIT_WATCH;
+						rc = audit_add_rule_data(fd,
+								rule_new,
+								add, action);
+					} else {
+						fprintf(stderr,
+				"Error sending add rule data request (%s)",
+						errno == EEXIST ?
+						"Rule exists" : strerror(-rc));
+					}
+				}
 			}
 		}
 		else if (del != AUDIT_FILTER_UNSET) {
@@ -1031,9 +1066,28 @@ static int handle_request(int status)
 			}
 			if (which == OLD)
 				rc = audit_delete_rule(fd, &rule, del, action);
-			else
+			else {
+				set_aumessage_mode(MSG_QUIET, DBG_NO);
 				rc = audit_delete_rule_data(fd, rule_new,
 								 del, action);
+				set_aumessage_mode(MSG_STDERR, DBG_NO);
+				/* Retry for legacy kernels */
+				if (rc < 0) {
+					if (errno == EINVAL &&
+					rule_new->fields[0] == AUDIT_DIR) {
+						rule_new->fields[0] =
+								AUDIT_WATCH;
+						rc = audit_delete_rule_data(fd,
+								rule_new,
+								del, action);
+					} else {
+						fprintf(stderr,
+				"Error sending delete rule data request (%s)",
+						errno == EEXIST ?
+						"Rule exists" : strerror(-rc));
+					}
+				}
+			}
 		} else {
         		usage();
 	    		audit_close(fd);
@@ -1178,6 +1232,12 @@ static int audit_print_reply(struct audit_reply *rep)
 						  &rep->ruledata->buf[boffset]);
 						boffset +=
 						    rep->ruledata->values[i];
+					} else if (field == AUDIT_DIR) {
+						printf(" dir=%.*s", 
+						  rep->ruledata->values[i],
+						  &rep->ruledata->buf[boffset]);
+						boffset +=
+						    rep->ruledata->values[i];
 					} else if (field == AUDIT_FILTERKEY) {
 						printf(" key=%.*s",
 						rep->ruledata->values[i],
