@@ -1,5 +1,5 @@
 /* auditd-reconfig.c -- 
- * Copyright 2005-06 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -99,17 +99,24 @@ static void *config_thread_main(void *arg)
 		/* We will re-use the current reply */
 		new_config.sender_uid = rep->reply.signal_info->uid;
 		new_config.sender_pid = rep->reply.signal_info->pid;
+		if (rep->reply.len > 24)
+			new_config.sender_ctx = 
+				strdup(rep->reply.signal_info->ctx);
+		else
+			new_config.sender_ctx = strdup("?"); 
 		memcpy(rep->reply.msg.data, &new_config, sizeof(new_config));
 		rep->reply.conf = (struct daemon_conf *)rep->reply.msg.data;
 		rep->reply.type = AUDIT_DAEMON_RECONFIG;
 		enqueue_event(rep);
 	} else {
 		// need to send a failed event message
-		char txt[80];
+		char txt[MAX_AUDIT_MESSAGE_LENGTH];
 		snprintf(txt, sizeof(txt),
-		    "reconfig aborted, sending auid=%u pid=%d res=failed",
+	    "reconfig aborted, sending auid=%u pid=%d subj=%s res=failed",
 			rep->reply.signal_info->uid,
-			rep->reply.signal_info->pid);
+			rep->reply.signal_info->pid,
+			(rep->reply.len > 24) ? 
+				rep->reply.signal_info->ctx : "?");
 		send_audit_event(AUDIT_DAEMON_CONFIG, txt);
 		free_config(&new_config);
 		free(rep);
