@@ -33,7 +33,6 @@
 static void print_title_summary(void);
 static void print_title_detailed(void);
 static void do_summary_output(void);
-static void do_watch_summary_output(slist *sptr);
 static void do_file_summary_output(slist *sptr);
 static void do_string_summary_output(slist *sptr);
 static void do_user_summary_output(slist *sptr);
@@ -79,9 +78,21 @@ static void print_title_summary(void)
 			printf("total  obj\n");
 			printf("=================================\n");
 			break;
+		case RPT_MAC:
+			printf("MAC Summary Report\n");
+			printf("======================\n");
+			printf("total  type\n");
+			printf("======================\n");
+			break;
 		case RPT_CONFIG:
 			UNIMPLEMENTED;
 			break;
+		case RPT_AUTH:
+			printf("Authentication Summary Report\n");
+			printf("=============================\n");
+			printf("total  acct\n");
+			printf("=============================\n");
+			break;
 		case RPT_LOGIN:
 			printf("Login Summary Report\n");
 			printf("============================\n");
@@ -142,12 +153,6 @@ static void print_title_summary(void)
 			printf("total  file\n");
 			printf("=================================\n");
 			break;
-		case RPT_WATCH:
-			printf("Watch Summary Report\n");
-			printf("============================\n");
-			printf("total  key  file\n");
-			printf("============================\n");
-			break;
 		case RPT_ANOMALY:
 			printf("Anomaly Summary Report\n");
 			printf("======================\n");
@@ -160,6 +165,12 @@ static void print_title_summary(void)
 			printf("total  type\n");
 			printf("===============================\n");
 			break;
+		case RPT_CRYPTO:
+			printf("Crypto Summary Report\n");
+			printf("======================\n");
+			printf("total  type\n");
+			printf("======================\n");
+			break;
 		default:
 			break;
 	}
@@ -172,17 +183,26 @@ static void print_title_detailed(void)
 		case RPT_AVC:
 			printf("AVC Report\n");
 			printf(
-			  "==============================================\n");
+		  "========================================================\n");
 			printf(
 		  "# date time comm subj syscall class permission obj event\n");
 			printf(
-			  "==============================================\n");
+		  "========================================================\n");
 			break;
 		case RPT_CONFIG:
 			printf("Config Change Report\n");
-			printf("==============================\n");
-			printf("# date time auid success event\n");
-			printf("==============================\n");
+			printf("===================================\n");
+			printf("# date time type auid success event\n");
+			printf("===================================\n");
+			break;
+		case RPT_AUTH:
+			printf("Authentication Report\n");
+			printf(
+			  "============================================\n");
+			printf(
+			  "# date time acct host term exe success event\n");
+			printf(
+			  "============================================\n");
 			break;
 		case RPT_LOGIN:
 			printf("Login Report\n");
@@ -209,9 +229,9 @@ static void print_title_detailed(void)
 		case RPT_EVENT:
 			if (report_detail == D_DETAILED) {
 				printf("Event Report\n");
-				printf("===========================\n");
-				printf("# date time event type auid\n");
-				printf("===========================\n");
+				printf("===================================\n");
+				printf("# date time event type auid success\n");
+				printf("===================================\n");
 			} else {
 				printf("Specific Event Report\n");
 				printf("=====================\n");
@@ -298,20 +318,6 @@ static void print_title_detailed(void)
 				printf("=======================\n");
 			}
 			break;
-		case RPT_WATCH:
-			if (report_detail == D_DETAILED) {
-			    printf("Watch Report\n");
-			    printf(
-		"====================================================\n");
-			    printf(
-		"# date time watch key syscall success exe auid event\n");
-			    printf(
-		"====================================================\n");
-			} else {
-				printf("Specific Watch Report\n");
-				printf("=====================\n");
-			}
-			break;
 		case RPT_EXE:
 			if (report_detail == D_DETAILED) {
 				printf("Executable Report\n");
@@ -351,6 +357,28 @@ static void print_title_detailed(void)
 				printf("===================================\n");
 			}
 			break;
+		case RPT_MAC:
+			if (report_detail == D_DETAILED) {
+				printf("MAC Report\n");
+				printf("===================================\n");
+				printf("# date time auid type success event\n");
+				printf("===================================\n");
+			} else {
+				printf("Specific Mandatory Access Control (MAC) Report\n");
+				printf("===================================\n");
+			}
+			break;
+		case RPT_CRYPTO:
+			if (report_detail == D_DETAILED) {
+				printf("Crypto Report\n");
+				printf("===================================\n");
+				printf("# date time auid type success event\n");
+				printf("===================================\n");
+			} else {
+				printf("Specific Crypto Report\n");
+				printf("===================================\n");
+			}
+			break;
 		default:
 			break;
 	}
@@ -363,29 +391,51 @@ void print_per_event_item(llist *l)
 	char date[32];
 	struct tm *tv;
 
-	line_item++;
 	// The beginning is common to all reports
 	tv = localtime(&l->e.sec);
-	strftime(date, sizeof(date), "%x %X", tv);
-	printf("%u. %s ", line_item, date);
+	strftime(date, sizeof(date), "%x %T", tv);
+	if (report_type != RPT_AVC) {
+		line_item++;
+		printf("%u. %s ", line_item, date);
+	}
 
 	switch (report_type)
 	{
 		case RPT_AVC:
+			alist_find_avc(l->s.avc);
+			do {
+				anode *an = l->s.avc->cur;
+				line_item++;
+				printf("%u. %s ", line_item, date);
 		// command subject syscall action obj res event
 			printf("%s %s %s %s %s %s %s %lu\n", 
 				l->s.comm ? l->s.comm : "?",
-				l->s.scontext, 
+				an->scontext, 
 				aulookup_syscall(l, buf,sizeof(buf)),
-				l->s.avc_class, l->s.avc_perm,
-				l->s.tcontext, aulookup_result(l->s.avc_result),
+				an->avc_class, an->avc_perm,
+				an->tcontext, aulookup_result(an->avc_result),
 				l->e.serial);
+//printf("items:%d\n", l->s.avc->cnt);
+			} while (alist_next_avc(l->s.avc));
 			break;
 		case RPT_CONFIG:
 			// FIXME:who, action, what, outcome, event
-			// NOW: auid success event
-			printf("%s success %lu\n",
+			// NOW: type auid success event
+			printf("%s %s %s %lu\n",
+				audit_msg_type_to_name(l->head->type),
 				aulookup_uid(l->s.loginuid, name, sizeof(name)),
+				aulookup_success(l->s.success), l->e.serial);
+			break;
+		case RPT_AUTH:
+			// who, addr, terminal, exe, success, event
+			// Special note...uid is used here because that is
+			// the way that the message works. This is because
+			// on failed logins, loginuid is not set.
+			printf("%s %s %s %s %s %lu\n",
+				l->s.acct ? l->s.acct :
+				aulookup_uid(l->s.uid, name, sizeof(name)),
+				l->s.hostname, l->s.terminal,
+				l->s.exe, aulookup_success(l->s.success),
 				l->e.serial);
 			break;
 		case RPT_LOGIN:
@@ -393,7 +443,9 @@ void print_per_event_item(llist *l)
 			// Special note...uid is used here because that is
 			// the way that the message works. This is because
 			// on failed logins, loginuid is not set.
-			printf("%s %s %s %s %s %lu\n",
+			printf("%s %s %s %s %s %lu\n", 
+				((l->s.success == S_FAILED) && l->s.acct) ?
+				l->s.acct :
 				aulookup_uid(l->s.uid, name, sizeof(name)),
 				l->s.hostname, l->s.terminal,
 				l->s.exe, aulookup_success(l->s.success),
@@ -410,12 +462,13 @@ void print_per_event_item(llist *l)
 				l->e.serial);
 			break;
 		case RPT_EVENT:	// report_detail == D_DETAILED
-			//        event, type, who
-			printf("%lu %s %s\n",
+			//        event, type, who, success
+			printf("%lu %s %s %s\n",
 				l->e.serial,
 				audit_msg_type_to_name(l->head->type),
 				aulookup_uid(l->s.loginuid, name, 
-						sizeof(name)));
+					sizeof(name)),
+					aulookup_success(l->s.success));
 			break;
 		case RPT_FILE:	// report_detail == D_DETAILED
 			// file, syscall, success, exe, who, event
@@ -471,19 +524,6 @@ void print_per_event_item(llist *l)
 				l->s.exe ? l->s.exe : "?",
 				l->e.serial);
 			break;
-		case RPT_WATCH:	// report_detail == D_DETAILED
-			// watch, syscall, who, event
-			slist_first(l->s.filename);
-			printf("%s %s %s %s %s %s %lu\n",
-				l->s.filename->cur->str,
-				l->s.filename->cur->key ? 
-					l->s.filename->cur->key : "<none>",
-				aulookup_syscall(l,buf,sizeof(buf)),
-				aulookup_success(l->s.success),
-                                l->s.exe ? l->s.exe : "?",
-				aulookup_uid(l->s.loginuid, name, sizeof(name)),
-				l->e.serial);
-			break;
 		case RPT_EXE:	// report_detail == D_DETAILED
 			// exe, terminal, host, who, event
 			printf("%s %s %s %s %lu\n",
@@ -497,7 +537,7 @@ void print_per_event_item(llist *l)
 			// type exe term host auid event
 			printf("%s %s %s %s %s %lu\n",
 				audit_msg_type_to_name(l->head->type),
-				l->s.exe ? l->s.exe : "?", 
+				l->s.exe ? l->s.exe : l->s.comm ? l->s.comm: "?", 
 				l->s.terminal ? l->s.terminal : "?",
 				l->s.hostname ? l->s.hostname : "?",
 				aulookup_uid(l->s.loginuid, name, sizeof(name)),
@@ -510,6 +550,22 @@ void print_per_event_item(llist *l)
 				aulookup_success(l->s.success),
 				l->e.serial);
 			break;
+		case RPT_MAC:
+			// auid type success event
+			printf("%s %s %s %lu\n",
+				aulookup_uid(l->s.loginuid, name, sizeof(name)),
+				audit_msg_type_to_name(l->head->type),
+				aulookup_success(l->s.success),
+				l->e.serial);
+			break;
+		case RPT_CRYPTO:
+			// auid type success event
+			printf("%s %s %s %lu\n",
+				aulookup_uid(l->s.loginuid, name, sizeof(name)),
+				audit_msg_type_to_name(l->head->type),
+				aulookup_success(l->s.success),
+				l->e.serial);
+			break;
 		default:
 			break;
 	}
@@ -531,6 +587,10 @@ void print_wrap_up(void)
 			break;
 		case RPT_CONFIG:
 			break;
+		case RPT_AUTH:
+			slist_sort_by_hits(&sd.users);
+			do_user_summary_output(&sd.users);
+			break;
 		case RPT_LOGIN:
 			slist_sort_by_hits(&sd.users);
 			do_user_summary_output(&sd.users);
@@ -569,10 +629,6 @@ void print_wrap_up(void)
 			slist_sort_by_hits(&sd.exes);
 			do_file_summary_output(&sd.exes);
 			break;
-		case RPT_WATCH:
-			slist_sort_by_hits(&sd.files);
-			do_watch_summary_output(&sd.files);
-			break;
 		case RPT_ANOMALY:
 			ilist_sort_by_hits(&sd.anom_list);
 			do_type_summary_output(&sd.anom_list);
@@ -581,6 +637,14 @@ void print_wrap_up(void)
 			ilist_sort_by_hits(&sd.resp_list);
 			do_type_summary_output(&sd.resp_list);
 			break;
+		case RPT_MAC:
+			ilist_sort_by_hits(&sd.mac_list);
+			do_type_summary_output(&sd.mac_list);
+			break;
+		case RPT_CRYPTO:
+			ilist_sort_by_hits(&sd.crypto_list);
+			do_type_summary_output(&sd.crypto_list);
+			break;
 		default:
 			break;
 	}
@@ -591,10 +655,10 @@ static void do_summary_output(void)
 	extern event very_first_event;
 	extern event very_last_event;
 
-	printf("Range of time: ");
+	printf("Range of time in logs: ");
 	{
 		struct tm *btm;
-		char tmp[32];
+		char tmp[48];
 
 		btm = localtime(&very_first_event.sec);
 		strftime(tmp, sizeof(tmp), "%x %T", btm);
@@ -603,43 +667,50 @@ static void do_summary_output(void)
 		strftime(tmp, sizeof(tmp), "%x %T", btm);
 		printf("%s.%03d\n", tmp, very_last_event.milli);
 	}
+	printf("Selected time for report: ");
+	{
+		struct tm *btm;
+		char tmp[48];
+
+		if (start_time)
+			btm = localtime(&start_time);
+		else
+			btm = localtime(&very_first_event.sec);
+		strftime(tmp, sizeof(tmp), "%x %T", btm);
+		printf("%s - ", tmp);
+		if (end_time) 
+			btm = localtime(&end_time);
+		else 
+			btm = localtime(&very_last_event.sec);
+		strftime(tmp, sizeof(tmp), "%x %T", btm);
+		if (end_time) 
+			printf("%s\n", tmp);
+		else 
+			printf("%s.%03d\n", tmp, very_last_event.milli);
+	}
 	printf("Number of changes in configuration: %lu\n", sd.changes);
-	printf("Number of changes to accounts or groups: %lu\n",
+	printf("Number of changes to accounts, groups, or roles: %lu\n",
 							 sd.acct_changes);
 	printf("Number of logins: %lu\n", sd.good_logins);
 	printf("Number of failed logins: %lu\n", sd.bad_logins);
+	printf("Number of authentications: %lu\n", sd.good_auth);
+	printf("Number of failed authentications: %lu\n", sd.bad_auth);
 	printf("Number of users: %u\n", sd.users.cnt);
 	printf("Number of terminals: %u\n", sd.terms.cnt);
 	printf("Number of host names: %u\n", sd.hosts.cnt);
 	printf("Number of executables: %u\n", sd.exes.cnt);
 	printf("Number of files: %u\n", sd.files.cnt);
-	printf("Number of AVC denials: %lu\n", sd.avcs);
+	printf("Number of AVC's: %lu\n", sd.avcs);
+	printf("Number of MAC events: %lu\n", sd.mac);
 	printf("Number of failed syscalls: %lu\n", sd.failed_syscalls);
-	printf("Number of watched file events: %lu\n", sd.fs_watches);
 	printf("Number of anomaly events: %lu\n", sd.anomalies);
 	printf("Number of responses to anomaly events: %lu\n", sd.responses);
+	printf("Number of crypto events: %lu\n", sd.crypto);
 	printf("Number of process IDs: %u\n", sd.pids.cnt);
 	printf("Number of events: %lu\n", sd.events);
 	printf("\n");
 }
 
-static void do_watch_summary_output(slist *sptr)
-{
-	const snode *sn;
-
-	if (sptr->cnt == 0) {
-		printf("<no events of interest were found>\n\n");
-		return;
-	}
-	slist_first(sptr);
-	sn=slist_get_cur(sptr);
-	while (sn) {
-		printf("%u  %s  %s\n", sn->hits, sn->key ? sn->key : "<none>",
-			sn->str);
-		sn=slist_next(sptr);
-	}
-}
-
 static void do_file_summary_output(slist *sptr)
 {
 	const snode *sn;
@@ -747,7 +818,7 @@ static void do_type_summary_output(ilist *sptr)
 	in=ilist_get_cur(sptr);
 	while (in) {
 		const char *name = audit_msg_type_to_name(in->num);
-		if (report_format == RPT_RAW)
+		if (report_format == RPT_DEFAULT)
 			printf("%u  %d\n", in->hits, in->num);
 		else
 			printf("%u  %s\n", in->hits, name);
