@@ -188,6 +188,12 @@ int scan(llist *l)
 		if (end_time == 0 || l->e.sec <= end_time) {
 			// OK - do the heavier checking
 			if (extract_search_items(l) == 0) {
+                                if (event_node) {
+                                        if (l->s.node == NULL)
+                                                return 0;
+                                        if (strcasecmp(event_node, l->s.node))
+                                                return 0;
+                                }
 				if (classify_success(l) && classify_conf(l))
 					return 1;
 				return 0;
