@@ -51,13 +51,13 @@ const char *aulookup_success(int s)
 {
 	switch (s)
 	{
-		case -1:
+		default:
 			return success[0];
 			break;
-		case 0:
+		case S_FAILED:
 			return success[1];
 			break;
-		default:
+		case S_SUCCESS:
 			return success[2];
 			break;
 	}
@@ -67,7 +67,7 @@ const char *aulookup_syscall(llist *l, char *buf, size_t size)
 {
 	const char *sys;
 
-	if (report_format == RPT_RAW) {
+	if (report_format <= RPT_DEFAULT) {
 		snprintf(buf, size, "%d", l->s.syscall);
 		return buf;
 	}
@@ -176,7 +176,7 @@ const char *aulookup_uid(uid_t uid, char *buf, size_t size)
 	char *name = NULL;
 	int rc;
 
-	if (report_format == RPT_RAW) {
+	if (report_format <= RPT_DEFAULT) {
 		snprintf(buf, size, "%d", uid);
 		return buf;
 	}
@@ -230,7 +230,7 @@ const char *aulookup_gid(gid_t gid, char *buf, size_t size)
 	char *name = NULL;
 	int rc;
 
-	if (report_format == RPT_RAW) {
+	if (report_format <= RPT_DEFAULT) {
 		snprintf(buf, size, "%d", gid);
 		return buf;
 	}
