@@ -1,5 +1,5 @@
 /* ausearch-options.c - parse commandline options and configure ausearch
- * Copyright 2005-06 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-07 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -48,6 +48,7 @@ uid_t event_uid = -1, event_euid = -1, event_loginuid = -1;
 int event_syscall = -1;
 int event_ua = 0, event_ga = 0, event_se = 0;
 const char *event_key = NULL;
+const char *event_node = NULL;
 const char *event_filename = NULL;
 const char *event_exe = NULL;
 const char *event_comm = NULL;
@@ -67,7 +68,7 @@ enum { S_EVENT, S_COMM, S_FILENAME, S_ALL_GID, S_EFF_GID, S_GID, S_HELP,
 S_HOSTNAME, S_INTERP, S_INFILE, S_MESSAGE_TYPE, S_PID, S_SYSCALL, S_OSUCCESS,
 S_TIME_END, S_TIME_START, S_TERMINAL, S_ALL_UID, S_EFF_UID, S_UID, S_LOGINID,
 S_VERSION, S_EXACT_MATCH, S_EXECUTABLE, S_CONTEXT, S_SUBJECT, S_OBJECT,
-S_PPID, S_KEY, S_RAW };
+S_PPID, S_KEY, S_RAW, S_NODE };
 
 static struct nv_pair optiontab[] = {
 	{ S_EVENT, "-a" },
@@ -94,6 +95,8 @@ static struct nv_pair optiontab[] = {
 	{ S_KEY, "--key" },
 	{ S_MESSAGE_TYPE, "-m" },
 	{ S_MESSAGE_TYPE, "--message" },
+	{ S_NODE, "-n" },
+	{ S_NODE, "--node" },
 	{ S_OBJECT, "-o" },
 	{ S_OBJECT, "--object" },
 	{ S_PID, "-p" },
@@ -154,11 +157,12 @@ static void usage(void)
 	"\t-ge,--gid-effective <effective Group id>  search based on Effective\n\t\t\t\t\tgroup id\n"
 	"\t-gi,--gid <Group Id>\t\tsearch based on group id\n"
 	"\t-h,--help\t\t\thelp\n"
-	"\t-hn,--host <Host Name>\t\tsearch based on host name\n"
+	"\t-hn,--host <Host Name>\t\tsearch based on remote host name\n"
 	"\t-i,--interpret\t\t\tInterpret results to be human readable\n"
 	"\t-if,--input <Input File name>\tuse this file instead of current logs\n"
 	"\t-k,--key  <key string>\t\tsearch based on key field\n"
 	"\t-m,--message  <Message type>\tsearch based on message type\n"
+	"\t-n,--node  <Node name>\t\tsearch based on machine's name\n"
 	"\t-o,--object  <SE Linux Object context> search based on context of object\n"
 	"\t-p,--pid  <Process id>\t\tsearch based on process id\n"
 	"\t-pp,--ppid <Parent Process id>\tsearch based on parent process id\n"
@@ -562,6 +566,19 @@ int check_params(int count, char *vars[])
         	                retval = -1;
 			}
 			break;
+		case S_NODE:
+			if (!optarg) {
+				fprintf(stderr, 
+					"Argument is required for %s\n",
+					vars[c]);
+				retval = -1;
+			} else {
+				event_node = strdup(optarg);
+				if (event_node == NULL)
+					retval = -1;
+				c++;
+			}
+			break;
 		case S_SYSCALL:
 			if (!optarg) {
 				fprintf(stderr, 
@@ -671,32 +688,9 @@ int check_params(int count, char *vars[])
 					// Check against recognized words
 					int t = lookup_time(optarg);
 					if (t >= 0) {
-						struct tm d;
-						switch (t)
-						{
-							case T_NOW:
-							    set_tm_now(&d);
-								break;
-							case T_RECENT:
-							    set_tm_recent(&d);
-								break;
-							case T_TODAY:
-							    set_tm_today(&d);
-								break;
-							case T_YESTERDAY:
-							   set_tm_yesterday(&d);
-								break;
-							case T_THIS_WEEK:
-							   set_tm_this_week(&d);
-								break;
-							case T_THIS_MONTH:
-							  set_tm_this_month(&d);
-								break;
-							case T_THIS_YEAR:
-							   set_tm_this_year(&d);
-								break;
-						}
-						end_time = mktime(&d); 
+						if (ausearch_time_end(optarg,
+								NULL) != 0)
+							retval = -1;
 					} else if ( (strchr(optarg, ':')) == NULL) {
 						/* Only have date */
 						if (ausearch_time_end(optarg,
@@ -736,37 +730,14 @@ int check_params(int count, char *vars[])
 					// Check against recognized words
 					int t = lookup_time(optarg);
 					if (t >= 0) {
-						struct tm d;
-						switch (t)
-						{
-							case T_NOW:
-							    set_tm_now(&d);
-								break;
-							case T_RECENT:
-							    set_tm_recent(&d);
-								break;
-							case T_TODAY:
-							    set_tm_today(&d);
-								break;
-							case T_YESTERDAY:
-							   set_tm_yesterday(&d);
-								break;
-							case T_THIS_WEEK:
-							   set_tm_this_week(&d);
-								break;
-							case T_THIS_MONTH:
-							  set_tm_this_month(&d);
-								break;
-							case T_THIS_YEAR:
-							   set_tm_this_year(&d);
-								break;
-						}
-						start_time = mktime(&d); 
+						if (ausearch_time_start(optarg,
+							"00:00:00") != 0)
+							retval = -1;
 					}
 					else if ( strchr(optarg, ':') == NULL) {
 						/* Only have date */
 						if (ausearch_time_start(optarg,
-							"00:00:01") != 0)
+							"00:00:00") != 0)
 							retval = -1;
 					} else {
 						/* Only have time */
