@@ -1,6 +1,6 @@
 /*
 * ausearch-parse.c - Extract interesting fields and check for match
-* Copyright (c) 2005-06 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2005-07 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved. 
 *
 * This software may be freely redistributed and/or modified under the
@@ -137,14 +137,26 @@ int extract_search_items(llist *l)
 	if (n) {
 		do {
 			char *ptr, *str, *term;
+			// get node if its there
+			if ((strncmp(n->message, "node=", 5) == 0) &&
+							s->node == NULL) {
+				str = n->message;
+				ptr = str+5;
+				term = strchr(ptr, ' ');
+				if (term == NULL)
+					return -1;
+				*term = 0;
+				s->node = strdup(ptr);
+				*term = ' ';
+			}
 			// get type
 			str = strstr(n->message, "type=");
 			if (str == NULL)
-				return -1;
+				return -2;
 			ptr = str+5;
 			term = strchr(ptr, ' ');
 			if (term == NULL)
-				return -2;
+				return -3;
 			*term = 0;
 			n->type = audit_name_to_msg_type(ptr);
 			*term = ' ';
@@ -548,6 +560,8 @@ static int common_path_parser(search_items *s, char *path)
 		if (s->filename) {
 			// append
 			snode sn;
+			if (!isxdigit(path[0]))
+				return 5;
 			sn.str = unescape(path);
 			sn.key = NULL;
 			sn.hits = 1;
@@ -556,7 +570,7 @@ static int common_path_parser(search_items *s, char *path)
 				(sn.str[1] == '/')) && s->cwd) {
 				char *tmp = malloc(PATH_MAX);
 				if (tmp == NULL)
-					return 5;
+					return 6;
 				snprintf(tmp, PATH_MAX, "%s/%s", 
 					s->cwd, sn.str);
 				free(sn.str);
@@ -717,13 +731,34 @@ static int parse_user(const lnode *n, search_items *s)
 
 			term = ptr + 1;
 			if (*ptr == '"') {
-				while (term != '"')
+				while (*term != '"')
 					term++;
+				saved = *term;
 				*term = 0;
 				s->acct = strdup(str);
-				*term = '"';
-			} else 
-				s->acct = unescape(str);
+				*term = saved;
+			} else { 
+				/* Handle legacy accts */
+				char *end = term;
+				int legacy = 0;
+
+				while (*end != ' ') {
+					if (!isxdigit(*end)) {
+						legacy = 1;
+						break;
+					}
+					end++;
+				}
+				term = end;
+				if (!legacy)
+					s->acct = unescape(str);
+				else {
+					saved = *term;
+					*term = 0;
+					s->acct = strdup(str);
+					*term = saved;
+				}
+			}
 		}
 	}
 	mptr = term + 1;
@@ -869,9 +904,22 @@ static int parse_login(const lnode *n, search_items *s)
 	if (term)
 		*term = ' ';
 
-	// Login means the kernel changed loginuid, its always
-	// successful at this point.
-	s->success = S_SUCCESS;
+	// success
+	str = strstr(ptr, "res=");
+	if (str != NULL) {
+		ptr = str + 4;
+		term = strchr(ptr, ' ');
+		if (term)
+			*term = 0;
+		errno = 0;
+		s->success = strtoul(ptr, NULL, 10);
+		if (errno)
+			return 9;
+		if (term)
+			*term = ' ';
+	} else	// Assume older kernel where always successful
+		s->success = S_SUCCESS; 
+
 	return 0;
 }
 
