@@ -23,6 +23,7 @@
 
 #include "config.h"
 #include <stdio.h>
+#include <stdio_ext.h>
 #include <string.h>
 #include <stdlib.h>
 #include <getopt.h>
@@ -101,6 +102,8 @@ int main(int argc, char *argv[])
 	ilist_clear(event_type);
 	free(event_type);
 	free(user_file);
+	aulookup_destroy_uid_list();
+	aulookup_destroy_gid_list();
 	if (rc)
 		return rc;
 	if (!found) {
@@ -164,8 +167,6 @@ static int process_logs(void)
 		else
 			break;
 	} while (1);
-	aulookup_destroy_uid_list();
-	aulookup_destroy_gid_list();
 	free(filename);
 	free_config(&config);
 	return 0;
@@ -180,8 +181,9 @@ static int process_log_fd(void)
 	list_create(&entries);
 	do {
 		ret = get_record(&entries);
-		if ((ret < 0)||(entries.cnt == 0))
+		if ((ret < 0)||(entries.cnt == 0)) {
 			break;
+		}
 		if (match(&entries)) {
 			output_record(&entries);
 			found = 1;
@@ -209,6 +211,7 @@ static int process_file(char *filename)
 		return 1;
 	}
 
+	__fsetlocking(log_fd, FSETLOCKING_BYCALLER);
 	return process_log_fd();
 }
 
@@ -255,6 +258,8 @@ static int get_record(llist *l)
 			if (ptr)
 				*ptr = 0;
 			n.message=strdup(buff);
+			// FIXME: need to extract the node here
+			// and put things on a list of lists
 			extract_timestamp(buff, &e);
 			if (first_time) {
 				l->e.milli = e.milli;
@@ -267,6 +272,7 @@ static int get_record(llist *l)
 			} else {
 				saved_buff = buff;
 				free(n.message);
+				buff = NULL;
 				break;
 			}
 		} else {
@@ -289,9 +295,12 @@ static void extract_timestamp(const char *b, event *e)
 {
 	char *ptr, *tmp;
 
-	tmp = strndupa(b, 80);
+	tmp = strndupa(b, 120);
 	ptr = strtok(tmp, " ");
 	if (ptr) {
+		while (ptr && strncmp(ptr, "type=", 5))
+			ptr = strtok(NULL, " ");
+
 		// at this point we have type=
 		ptr = strtok(NULL, " ");
 		if (ptr) {
